@@ -31,11 +31,7 @@
  * SUCH DAMAGE.
  */
 
-#ifndef lint
-static char sccsid[] = "@(#)sys_bsd.c	8.4 (Berkeley) 5/30/95";
-#endif /* not lint */
-
-#include <config.h>
+/* based on @(#)sys_bsd.c	8.1 (Berkeley) 6/6/93 */
 
 /*
  * The following routines try to encapsulate what is system dependent
@@ -45,12 +41,15 @@ static char sccsid[] = "@(#)sys_bsd.c	8.4 (Berkeley) 5/30/95";
 
 #include <fcntl.h>
 #include <sys/types.h>
+#include <sys/time.h>
+#include <sys/socket.h>
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif
-#include <sys/time.h>
-#include <sys/socket.h>
 #include <signal.h>
+#ifdef POSIX_SIGNALS
+#include <unistd.h>
+#endif /* POSIX_SIGNALS */
 #include <errno.h>
 #include <arpa/telnet.h>
 
@@ -68,10 +67,40 @@ static char sccsid[] = "@(#)sys_bsd.c	8.4 (Berkeley) 5/30/95";
 #define	SIG_FUNC_RET	int
 #endif
 
+#ifdef	SIGTSTP
+static SIG_FUNC_RET susp(int);
+#endif	/* SIGTSTP */
+#ifdef	SIGINFO
+SIG_FUNC_RET ayt(int);
+#endif
 #ifdef	SIGINFO
 extern SIG_FUNC_RET ayt_status();
 #endif
 
+#ifdef POSIX_SIGNALS
+static struct sigaction new_sa_rec, old_sa_rec;
+
+#ifdef SA_INTERRUPT
+#define SIGACTION_INTERRUPT SA_INTERRUPT
+#else
+#define SIGACTION_INTERRUPT 0
+#endif
+
+#ifdef SA_NOMASK
+#define SIGACTION_NOMASK SA_NOMASK
+#else
+#define SIGACTION_NOMASK 0
+#endif
+
+#define signal(sig, func) ((new_sa_rec.sa_handler = func), \
+			   sigemptyset(&new_sa_rec.sa_mask), \
+			   (new_sa_rec.sa_flags = SIGACTION_INTERRUPT | \
+			    SIGACTION_NOMASK), \
+			   sigaction(sig, &new_sa_rec, &old_sa_rec), \
+			   old_sa_rec.sa_handler)
+
+#endif /* POSIX_SIGNALS */
+
 int
 	tout,			/* Output file descriptor */
 	tin,			/* Input file descriptor */
@@ -137,7 +166,7 @@ init_sys()
 
     int
 TerminalWrite(buf, n)
-    char *buf;
+    unsigned char *buf;
     int  n;
 {
     return write(tout, buf, n);
@@ -145,7 +174,7 @@ TerminalWrite(buf, n)
 
     int
 TerminalRead(buf, n)
-    char *buf;
+    unsigned char *buf;
     int  n;
 {
     return read(tin, buf, n);
@@ -183,8 +212,6 @@ extern int kludgelinemode;
  *	1	Do add this character
  */
 
-extern void xmitAO(), xmitEL(), xmitEC(), intp(), sendbrk();
-
     int
 TerminalSpecialChars(c)
     int	c;
@@ -228,7 +255,6 @@ TerminalSpecialChars(c)
 /*
  * Flush output to the terminal
  */
-
     void
 TerminalFlushOutput()
 {
@@ -333,7 +359,7 @@ TerminalDefaultChars()
     nttyb.sg_kill = ottyb.sg_kill;
     nttyb.sg_erase = ottyb.sg_erase;
 #else	/* USE_TERMIO */
-    memmove(new_tc.c_cc, old_tc.c_cc, sizeof(old_tc.c_cc));
+    memcpy(new_tc.c_cc, old_tc.c_cc, sizeof(old_tc.c_cc));
 # ifndef	VDISCARD
     termFlushChar = CONTROL('O');
 # endif
@@ -615,12 +641,6 @@ TerminalNewMode(f)
     }
 
     if (f != -1) {
-#ifdef	SIGTSTP
-	SIG_FUNC_RET susp();
-#endif	/* SIGTSTP */
-#ifdef	SIGINFO
-	SIG_FUNC_RET ayt();
-#endif
 
 #ifdef	SIGTSTP
 	(void) signal(SIGTSTP, susp);
@@ -674,11 +694,16 @@ TerminalNewMode(f)
 #endif
 #ifdef	SIGTSTP
 	(void) signal(SIGTSTP, SIG_DFL);
-# ifndef SOLARIS
+#ifdef POSIX_SIGNALS
+	{
+	  sigset_t tmask;
+	  sigemptyset(&tmask);
+	  sigaddset(&tmask, SIGTSTP);
+	  sigprocmask(SIG_UNBLOCK, &tmask, (sigset_t*)0);
+	}
+#else
 	(void) sigsetmask(sigblock(0) & ~(1<<(SIGTSTP-1)));
-# else
-	(void) sigrelse(SIGTSTP);
-# endif
+#endif
 #endif	/* SIGTSTP */
 #ifndef USE_TERMIO
 	ltc = oltc;
@@ -713,51 +738,14 @@ TerminalNewMode(f)
 
 }
 
-/*
- * Try to guess whether speeds are "encoded" (4.2BSD) or just numeric (4.4BSD).
- */
-#if B4800 != 4800
-#define	DECODE_BAUD
-#endif
-
-#ifdef	DECODE_BAUD
-#ifndef	B7200
-#define B7200   B4800
-#endif
-
-#ifndef	B14400
-#define B14400  B9600
-#endif
-
 #ifndef	B19200
-# define B19200 B14400
-#endif
-
-#ifndef	B28800
-#define B28800  B19200
+# define B19200 B9600
 #endif
 
 #ifndef	B38400
-# define B38400 B28800
+# define B38400 B19200
 #endif
 
-#ifndef B57600
-#define B57600  B38400
-#endif
-
-#ifndef B76800
-#define B76800  B57600
-#endif
-
-#ifndef B115200
-#define B115200 B76800
-#endif
-
-#ifndef B230400
-#define B230400 B115200
-#endif
-
-
 /*
  * This code assumes that the values B0, B50, B75...
  * are in ascending order.  They do not have to be
@@ -767,25 +755,20 @@ struct termspeeds {
 	long speed;
 	long value;
 } termspeeds[] = {
-	{ 0,      B0 },      { 50,    B50 },    { 75,     B75 },
-	{ 110,    B110 },    { 134,   B134 },   { 150,    B150 },
-	{ 200,    B200 },    { 300,   B300 },   { 600,    B600 },
-	{ 1200,   B1200 },   { 1800,  B1800 },  { 2400,   B2400 },
-	{ 4800,   B4800 },   { 7200,  B7200 },  { 9600,   B9600 },
-	{ 14400,  B14400 },  { 19200, B19200 }, { 28800,  B28800 },
-	{ 38400,  B38400 },  { 57600, B57600 }, { 115200, B115200 },
-	{ 230400, B230400 }, { -1,    B230400 }
+	{ 0,     B0 },     { 50,    B50 },   { 75,    B75 },
+	{ 110,   B110 },   { 134,   B134 },  { 150,   B150 },
+	{ 200,   B200 },   { 300,   B300 },  { 600,   B600 },
+	{ 1200,  B1200 },  { 1800,  B1800 }, { 2400,  B2400 },
+	{ 4800,  B4800 },  { 9600,  B9600 }, { 19200, B19200 },
+	{ 38400, B38400 }, { -1,    B38400 }
 };
-#endif	/* DECODE_BAUD */
 
     void
 TerminalSpeeds(ispeed, ospeed)
     long *ispeed;
     long *ospeed;
 {
-#ifdef	DECODE_BAUD
     register struct termspeeds *tp;
-#endif	/* DECODE_BAUD */
     register long in, out;
 
     out = cfgetospeed(&old_tc);
@@ -793,7 +776,6 @@ TerminalSpeeds(ispeed, ospeed)
     if (in == 0)
 	in = out;
 
-#ifdef	DECODE_BAUD
     tp = termspeeds;
     while ((tp->speed != -1) && (tp->value < in))
 	tp++;
@@ -803,10 +785,6 @@ TerminalSpeeds(ispeed, ospeed)
     while ((tp->speed != -1) && (tp->value < out))
 	tp++;
     *ospeed = tp->speed;
-#else	/* DECODE_BAUD */
-	*ispeed = in;
-	*ospeed = out;
-#endif	/* DECODE_BAUD */
 }
 
     int
@@ -833,7 +811,7 @@ NetClose(fd)
 }
 
 
-    void
+static void
 NetNonblockingIO(fd, onoff)
     int fd;
     int onoff;
@@ -866,7 +844,7 @@ NetSetPgrp(fd)
  */
 
     /* ARGSUSED */
-    SIG_FUNC_RET
+static  SIG_FUNC_RET
 deadpeer(sig)
     int sig;
 {
@@ -874,11 +852,18 @@ deadpeer(sig)
 	longjmp(peerdied, -1);
 }
 
+int intr_happened = 0;
+int intr_waiting = 0;
+
     /* ARGSUSED */
-    SIG_FUNC_RET
+static SIG_FUNC_RET
 intr(sig)
     int sig;
 {
+    if (intr_waiting) {
+	intr_happened = 1;
+	return;
+    }
     if (localchars) {
 	intp();
 	return;
@@ -888,7 +873,7 @@ intr(sig)
 }
 
     /* ARGSUSED */
-    SIG_FUNC_RET
+static SIG_FUNC_RET
 intr2(sig)
     int sig;
 {
@@ -905,7 +890,7 @@ intr2(sig)
 
 #ifdef	SIGTSTP
     /* ARGSUSED */
-    SIG_FUNC_RET
+static SIG_FUNC_RET
 susp(sig)
     int sig;
 {
@@ -918,7 +903,7 @@ susp(sig)
 
 #ifdef	SIGWINCH
     /* ARGSUSED */
-    SIG_FUNC_RET
+static  SIG_FUNC_RET
 sendwin(sig)
     int sig;
 {
@@ -1002,7 +987,7 @@ process_rings(netin, netout, netex, ttyin, ttyout, poll)
 
     if (netout) {
 	FD_SET(net, &obits);
-    }
+    } 
     if (ttyout) {
 	FD_SET(tout, &obits);
     }
@@ -1141,7 +1126,7 @@ process_rings(netin, netout, netex, ttyin, ttyout, poll)
 		    int i;
 		    i = recv(net, netiring.supply + c, canread - c, MSG_OOB);
 		    if (i == c &&
-			 memcmp(netiring.supply, netiring.supply + c, i) == 0) {
+			memcmp(netiring.supply, netiring.supply + c, i) == 0) {
 			bogus_oob = 1;
 			first = 0;
 		    } else if (i < 0) {
@@ -1190,8 +1175,6 @@ process_rings(netin, netout, netex, ttyin, ttyout, poll)
     if (FD_ISSET(tin, &ibits)) {
 	FD_CLR(tin, &ibits);
 	c = TerminalRead(ttyiring.supply, ring_empty_consecutive(&ttyiring));
-	if (c < 0 && errno == EIO)
-	    c = 0;
 	if (c < 0 && errno == EWOULDBLOCK) {
 	    c = 0;
 	} else {
