@@ -42,6 +42,9 @@ static char copyright[] =
 #include "telnetd.h"
 #include "pathnames.h"
 
+extern int getent(char *, char *);
+extern int tgetent(char *, char *);
+
 #if	defined(_SC_CRAY_SECURE_SYS) && !defined(SCM_SECURITY)
 /*
  * UNICOS 6.0/6.1 do not have SCM_SECURITY defined, so we can
@@ -74,6 +77,8 @@ struct	socket_security ss;
 # endif /* SO_SEC_MULTI */
 #endif	/* _SC_CRAY_SECURE_SYS */
 
+#include "fake-addrinfo.h"
+
 #ifdef KRB5
 #include "krb5.h"
 #endif
@@ -86,6 +91,9 @@ struct	socket_security ss;
 #include <libtelnet/encrypt.h>
 #include <libtelnet/enc-proto.h>
 #endif
+#if	defined(AUTHENTICATION) || defined(ENCRYPTION)
+#include <libtelnet/misc-proto.h>
+#endif
 
 int	registerd_host_only = 0;
 
@@ -97,11 +105,10 @@ int	registerd_host_only = 0;
 #ifdef HAVE_SYS_TTY_H
 # include "/usr/include/sys/tty.h"
 #endif
-#ifdef  HAS_PTYVAR
+#ifdef  HAVE_SYS_PTYVAR_H
 # include <sys/ptyvar.h>
 #endif
 
-
 /*
  * Because of the way ptyibuf is used with streams messages, we need
  * ptyibuf+1 to be on a full-word boundary.  The following wierdness
@@ -127,7 +134,9 @@ char	ptyibuf2[BUFSIZ];
 
 #endif /* ! STREAMPTY */
 
-void doit P((struct sockaddr_in *));
+static void doit (struct sockaddr *);
+int terminaltypeok (char *);
+static void _gettermname(void);
 
 int	hostinfo = 1;			/* do we print login banner? */
 
@@ -144,7 +153,7 @@ int maxhostlen = 0;
 int always_ip = 0;
 int stripdomain = 1;
 
-extern void usage P((void));
+extern void usage (void);
 
 /*
  * The string to pass to getopt().  We do it this way so
@@ -153,7 +162,7 @@ extern void usage P((void));
  */
 char valid_opts[] = {
 	'd', ':', 'h', 'k', 'L', ':', 'n', 'S', ':', 'U',
-	'w',
+	'w', ':',
 #ifdef	AUTHENTICATION
 	'a', ':', 'X', ':',
 #endif
@@ -184,12 +193,36 @@ char valid_opts[] = {
 	'\0'
 };
 
+#include <sys/utsname.h>
+static char *
+get_default_IM()
+{
+	struct utsname name;
+	static char banner[1024];
+	
+	if (uname(&name) < 0)
+		sprintf(banner, "\r\nError getting hostname: %s\r\n",
+		    strerror(errno));
+        else {
+#if defined(_AIX)
+		sprintf(banner, "\r\n    %%h (%s release %s.%s) (%%t)\r\n\r\n",
+		    name.sysname, name.version, name.release);
+#else
+		sprintf(banner, "\r\n    %%h (%s release %s %s) (%%t)\r\n\r\n",
+		    name.sysname, name.release, name.version);
+#endif
+	}
+	return banner;
+}
+
+int
 main(argc, argv)
 	int argc;
 	char *argv[];
 {
-	struct sockaddr_in from;
-	int on = 1, fromlen;
+	struct sockaddr_storage from;
+	int on = 1;
+	socklen_t fromlen;
 	register int ch;
 	extern char *optarg;
 	extern int optind;
@@ -214,7 +247,7 @@ main(argc, argv)
 	highpty = getnpty();
 #endif /* CRAY */
 
-	while ((ch = getopt(argc, argv, valid_opts)) != EOF) {
+	while ((ch = getopt(argc, argv, valid_opts)) != -1) {
 		switch(ch) {
 
 #ifdef	AUTHENTICATION
@@ -387,7 +420,7 @@ main(argc, argv)
 			break;
 #endif	/* SecurID */
 		case 'S':
-#ifdef	HAS_GETTOS
+#ifdef	HAVE_GETTOSBYNAME
 			if ((tos = parsetos(optarg, "tcp")) < 0)
 				fprintf(stderr, "%s%s%s\n",
 					"telnetd: Bad TOS argument '", optarg,
@@ -442,16 +475,6 @@ main(argc, argv)
 				}
 			}
 			break;
-		case 'u':
-			maxhostlen = atoi(optarg);
-			break;
-		case 'i':
-			always_ip = 1;
-			break;
-		case 'N':
-			stripdomain = 0;
-			break;
-
 		default:
 			fprintf(stderr, "telnetd: %c: unknown option\n", ch);
 			/* FALLTHROUGH */
@@ -464,25 +487,27 @@ main(argc, argv)
 	argc -= optind;
 	argv += optind;
 
+	/* XXX Convert this to support getaddrinfo, ipv6, etc.  */
 	if (debug) {
-	    int s, ns, foo;
+	    int s, ns;
+	    socklen_t foo;
 	    struct servent *sp;
-	    static struct sockaddr_in sin = { AF_INET };
+	    static struct sockaddr_in sin4 = { AF_INET };
 
 	    if (argc > 1) {
 		usage();
 		/* NOT REACHED */
 	    } else if (argc == 1) {
 		    if ((sp = getservbyname(*argv, "tcp"))) {
-			sin.sin_port = sp->s_port;
+			sin4.sin_port = sp->s_port;
 		    } else {
-			sin.sin_port = atoi(*argv);
-			if ((int)sin.sin_port <= 0) {
+			sin4.sin_port = atoi(*argv);
+			if ((int)sin4.sin_port <= 0) {
 			    fprintf(stderr, "telnetd: %s: bad port #\n", *argv);
 			    usage();
 			    /* NOT REACHED */
 			}
-			sin.sin_port = htons((u_short)sin.sin_port);
+			sin4.sin_port = htons((u_short)sin4.sin_port);
 		   }
 	    } else {
 		sp = getservbyname("telnet", "tcp");
@@ -490,7 +515,7 @@ main(argc, argv)
 		    fprintf(stderr, "telnetd: tcp/telnet: unknown service\n");
 		    exit(1);
 		}
-		sin.sin_port = sp->s_port;
+		sin4.sin_port = sp->s_port;
 	    }
 
 	    s = socket(AF_INET, SOCK_STREAM, 0);
@@ -500,7 +525,7 @@ main(argc, argv)
 	    }
 	    (void) setsockopt(s, SOL_SOCKET, SO_REUSEADDR,
 				(char *)&on, sizeof(on));
-	    if (bind(s, (struct sockaddr *)&sin, sizeof sin) < 0) {
+	    if (bind(s, (struct sockaddr *)&sin4, sizeof(sin4)) < 0) {
 		perror("bind");
 		exit(1);
 	    }
@@ -508,8 +533,8 @@ main(argc, argv)
 		perror("listen");
 		exit(1);
 	    }
-	    foo = sizeof sin;
-	    ns = accept(s, (struct sockaddr *)&sin, &foo);
+	    foo = sizeof(sin4);
+	    ns = accept(s, (struct sockaddr *)&sin4, &foo);
 	    if (ns < 0) {
 		perror("accept");
 		exit(1);
@@ -585,6 +610,7 @@ main(argc, argv)
 
 	openlog("telnetd", LOG_PID | LOG_ODELAY, LOG_DAEMON);
 	fromlen = sizeof (from);
+	memset(&from, 0, sizeof(from));
 	if (getpeername(0, (struct sockaddr *)&from, &fromlen) < 0) {
 		fprintf(stderr, "%s: ", progname);
 		perror("getpeername");
@@ -597,8 +623,8 @@ main(argc, argv)
 	}
 
 #if	defined(IPPROTO_IP) && defined(IP_TOS)
-	{
-# if	defined(HAS_GETTOS)
+	if (fromlen == sizeof (struct in_addr)) {
+# if	defined(HAVE_GETTOSBYNAME)
 		struct tosent *tp;
 		if (tos < 0 && (tp = gettosbyname("telnet", "tcp")))
 			tos = tp->t_tos;
@@ -613,7 +639,7 @@ main(argc, argv)
 	}
 #endif	/* defined(IPPROTO_IP) && defined(IP_TOS) */
 	net = 0;
-	doit(&from);
+	doit((struct sockaddr *)&from);
 	
 	/* NOTREACHED */
 	return 0;
@@ -654,24 +680,24 @@ usage()
 #ifdef	SecurID
 	fprintf(stderr, " [-s]");
 #endif
-#ifdef	HAS_GETTOS
+#ifdef	HAVE_GETTOSBYNAME
 	fprintf(stderr, " [-S tos]");
 #endif
 #ifdef	AUTHENTICATION
 	fprintf(stderr, " [-X auth-type]");
 #endif
-	fprintf(stderr, " [-u utmp_hostname_length] [-U]\n");
-	fprintf(stderr, " [-w [ip|maxhostlen[,[no]striplocal]]]\n");
+	fprintf(stderr, " [-U]\n\t");
+	fprintf(stderr, " [-w [ip|maxhostlen[,[no]striplocal]]]\n\t");
 	fprintf(stderr, " [port]\n");
 	exit(1);
 }
 
 static void encrypt_failure()
 {
-    char *error_message =
+    char *lerror_message =
 	"Encryption was not successfully negotiated.  Goodbye.\r\n\r\n";
 
-    writenet(error_message, strlen(error_message));
+    netputs(lerror_message);
     netflush();
     exit(1);
 }
@@ -686,12 +712,11 @@ static unsigned char ttytype_sbbuf[] = {
 	IAC, SB, TELOPT_TTYPE, TELQUAL_SEND, IAC, SE
 };
 
-    int
+static int
 getterminaltype(name)
     char *name;
 {
     int retval = -1;
-    void _gettermname();
 
     settimer(baseline);
 #if	defined(AUTHENTICATION)
@@ -758,36 +783,26 @@ getterminaltype(name)
     if (his_state_is_will(TELOPT_TSPEED)) {
 	static unsigned char sb[] =
 			{ IAC, SB, TELOPT_TSPEED, TELQUAL_SEND, IAC, SE };
-
-	memcpy(nfrontp, sb, sizeof sb);
-	nfrontp += sizeof sb;
+	netwrite(sb, sizeof(sb));
     }
     if (his_state_is_will(TELOPT_XDISPLOC)) {
 	static unsigned char sb[] =
 			{ IAC, SB, TELOPT_XDISPLOC, TELQUAL_SEND, IAC, SE };
-
-	memcpy(nfrontp, sb, sizeof sb);
-	nfrontp += sizeof sb;
+	netwrite(sb, sizeof(sb));
     }
     if (his_state_is_will(TELOPT_NEW_ENVIRON)) {
 	static unsigned char sb[] =
 			{ IAC, SB, TELOPT_NEW_ENVIRON, TELQUAL_SEND, IAC, SE };
-
-	memcpy(nfrontp, sb, sizeof sb);
-	nfrontp += sizeof sb;
+	netwrite(sb, sizeof(sb));
     }
     else if (his_state_is_will(TELOPT_OLD_ENVIRON)) {
 	static unsigned char sb[] =
 			{ IAC, SB, TELOPT_OLD_ENVIRON, TELQUAL_SEND, IAC, SE };
-
-	memcpy(nfrontp, sb, sizeof sb);
-	nfrontp += sizeof sb;
+	netwrite(sb, sizeof(sb));
     }
-    if (his_state_is_will(TELOPT_TTYPE)) {
+    if (his_state_is_will(TELOPT_TTYPE))
+	netwrite(ttytype_sbbuf, sizeof(ttytype_sbbuf));
 
-	memcpy(nfrontp, ttytype_sbbuf, sizeof ttytype_sbbuf);
-	nfrontp += sizeof ttytype_sbbuf;
-    }
     if (his_state_is_will(TELOPT_TSPEED)) {
 	while (sequenceIs(tspeedsubopt, baseline))
 	    ttloop();
@@ -853,7 +868,7 @@ getterminaltype(name)
     return(retval);
 }  /* end of getterminaltype */
 
-    void
+static void
 _gettermname()
 {
     /*
@@ -864,8 +879,7 @@ _gettermname()
     if (his_state_is_wont(TELOPT_TTYPE))
 	return;
     settimer(baseline);
-    memcpy(nfrontp, ttytype_sbbuf, sizeof ttytype_sbbuf);
-    nfrontp += sizeof ttytype_sbbuf;
+    netwrite(ttytype_sbbuf, sizeof(ttytype_sbbuf));
     while (sequenceIs(ttypesubopt, baseline))
 	ttloop();
 }
@@ -904,27 +918,27 @@ char remote_host_name[MAXDNAME];
 char *rhost_sane;
 
 #ifndef	convex
-extern void telnet P((int, int));
+extern void telnet (int, int);
 #else
-extern void telnet P((int, int, char *));
+extern void telnet (int, int, char *);
 #endif
 
 /*
  * Get a pty, scan input lines.
  */
-void doit(who)
-	struct sockaddr_in *who;
+static void doit(who)
+	struct sockaddr *who;
 {
-	char *host, *inet_ntoa();
-	struct hostent *hp;
 	int level;
+#if	defined(_SC_CRAY_SECURE_SYS)
 	int ptynum;
+#endif
 	char user_name[256];
-long retval;
+	long retval;
 	/*
 	 * Find an available pty to use.
 	 */
-pty_init();
+	pty_init();
 	
 
 	if ((retval = pty_getpty(&pty, line, 17)) != 0) {
@@ -947,19 +961,25 @@ pty_init();
 	}
 #endif	/* _SC_CRAY_SECURE_SYS */
 
-	retval = pty_make_sane_hostname(who, maxhostlen,
+	retval = pty_make_sane_hostname((struct sockaddr *) who, maxhostlen,
 					stripdomain, always_ip,
 					&rhost_sane);
 	if (retval) {
 		fatal(net, error_message(retval));
 	}
-	/* get name of connected client */
-	hp = gethostbyaddr((char *)&who->sin_addr, sizeof (struct in_addr),
-		who->sin_family);
-
-	if (hp == NULL && registerd_host_only) {
-		fatal(net, "Couldn't resolve your address into a host name.\r\n\
-         Please contact your net administrator");
+	if (registerd_host_only) {
+	    /* Get name of connected client -- but we don't actually
+	       use it.  Just confirm that we can get it.  */
+	    int aierror;
+	    char hostnamebuf[NI_MAXHOST];
+	    aierror = getnameinfo (who, socklen (who),
+				   hostnamebuf, sizeof (hostnamebuf), 0, 0,
+				   NI_NAMEREQD);
+	    if (aierror != 0) {
+		fatal(net,
+		      "Couldn't resolve your address into a host name.\r\n"
+		      "Please contact your net administrator");
+	    }
 	}
 
 	(void) gethostname(host_name, sizeof (host_name));
@@ -986,6 +1006,12 @@ pty_init();
 	level = getterminaltype(user_name);
 	setenv("TERM", *terminaltype ? terminaltype : "network", 1);
 
+#if defined (AUTHENTICATION)
+	if (level < 0 && auth_level > 0) {
+		fatal (net, "No authentication provided");
+		exit (-1);
+	}
+#endif
 	/*
 	 * Start up the login process on the slave side of the terminal
 	 */
@@ -1131,9 +1157,7 @@ telnet(f, p, host)
 	 * mode, which we do not want.
 	 */
 	if (his_want_state_is_will(TELOPT_ECHO)) {
-		DIAG(TD_OPTIONS,
-			{sprintf(nfrontp, "td: simulating recv\r\n");
-			 nfrontp += strlen(nfrontp);});
+		DIAG(TD_OPTIONS, netputs("td: simulating recv\r\n"));
 		willoption(TELOPT_ECHO);
 	}
 
@@ -1177,7 +1201,7 @@ telnet(f, p, host)
 
 #if	defined(SO_OOBINLINE)
 	(void) setsockopt(net, SOL_SOCKET, SO_OOBINLINE,
-				(char *)&on, sizeof on);
+				(char *)&on, sizeof(on));
 #endif	/* defined(SO_OOBINLINE) */
 
 #ifdef	SIGTSTP
@@ -1238,11 +1262,12 @@ telnet(f, p, host)
 		HN = getstr("hn", &cp);
 		IM = getstr("im", &cp);
 		if (HN && *HN)
-			(void) strcpy(host_name, HN);
+			(void) strncpy(host_name, HN, sizeof(host_name) - 1);
+		host_name[sizeof(host_name) - 1] = '\0';
 		if (IM == 0)
 			IM = "";
 	} else {
-		IM = DEFAULT_IM;
+		IM = get_default_IM();
 		HEstr = 0;
 	}
 	edithost(HEstr, host_name);
@@ -1261,9 +1286,7 @@ telnet(f, p, host)
 	localstat();
 #endif	/* LINEMODE */
 
-	DIAG(TD_REPORT,
-		{sprintf(nfrontp, "td: Entering processing loop\r\n");
-		 nfrontp += strlen(nfrontp);});
+	DIAG(TD_REPORT, netputs("td: Entering processing loop\r\n"));
 
 #ifdef	convex
 	startslave(host);
@@ -1388,8 +1411,7 @@ telnet(f, p, host)
 			netip = netibuf;
 		    }
 		    DIAG((TD_REPORT | TD_NETDATA),
-			    {sprintf(nfrontp, "td: netread %d chars\r\n", ncc);
-			     nfrontp += strlen(nfrontp);});
+			 netprintf("td: netread %d chars\r\n", ncc));
 		    DIAG(TD_NETDATA, printdata("nd", netip, ncc));
 		}
 
@@ -1436,9 +1458,7 @@ telnet(f, p, host)
 					 * royally if we send them urgent
 					 * mode data.
 					 */
-					*nfrontp++ = IAC;
-					*nfrontp++ = DM;
-					neturg = nfrontp-1; /* off by one XXX */
+					netprintf_urg("%c%c", IAC, DM);
 #endif
 				}
 				if (his_state_is_will(TELOPT_LFLOW) &&
@@ -1448,13 +1468,11 @@ telnet(f, p, host)
 					    ptyibuf[0] & TIOCPKT_DOSTOP ? 1 : 0;
 					if (newflow != flowmode) {
 						flowmode = newflow;
-						(void) sprintf(nfrontp,
-							"%c%c%c%c%c%c",
+						netprintf("%c%c%c%c%c%c",
 							IAC, SB, TELOPT_LFLOW,
 							flowmode ? LFLOW_ON
 								 : LFLOW_OFF,
 							IAC, SE);
-						nfrontp += 6;
 					}
 				}
 				pcc--;
@@ -1477,19 +1495,19 @@ telnet(f, p, host)
 				break;
 			c = *ptyip++ & 0377, pcc--;
 			if (c == IAC)
-				*nfrontp++ = c;
+				netprintf("%c", c);
 #if	defined(CRAY2) && defined(UNICOS5)
 			else if (c == '\n' &&
 				     my_state_is_wont(TELOPT_BINARY) && newmap)
-				*nfrontp++ = '\r';
+				netputs("\r");
 #endif	/* defined(CRAY2) && defined(UNICOS5) */
-			*nfrontp++ = c;
+			netprintf("%c", c);
 			if ((c == '\r') && (my_state_is_wont(TELOPT_BINARY))) {
 				if (pcc > 0 && ((*ptyip & 0377) == '\n')) {
-					*nfrontp++ = *ptyip++ & 0377;
+					netprintf("%c", *ptyip++ & 0377);
 					pcc--;
 				} else
-					*nfrontp++ = '\0';
+					netprintf("%c", '\0');
 			}
 		}
 #if	defined(CRAY2) && defined(UNICOS5)
@@ -1511,6 +1529,7 @@ telnet(f, p, host)
 		if (FD_ISSET(p, &obits) && (pfrontp - pbackp) > 0)
 			ptyflush();
 	}
+	(void) signal(SIGCHLD, SIG_DFL);
 	cleanup(0);
 }  /* end of telnet */
 	
@@ -1650,7 +1669,7 @@ sendsusp()
  * When we get an AYT, if ^T is enabled, use that.  Otherwise,
  * just send back "[Yes]".
  */
-	void
+void
 recv_ayt()
 {
 #if	defined(SIGINFO) && defined(TCSIG)
@@ -1659,8 +1678,7 @@ recv_ayt()
 		return;
 	}
 #endif
-	(void) strcpy(nfrontp, "\r\n[Yes]\r\n");
-	nfrontp += 9;
+	netputs("\r\n[Yes]\r\n");
 }
 
 	void
