@@ -1,13 +1,8 @@
 /*
  * Copyright 1993 OpenVision Technologies, Inc., All Rights Reserved
  *
- * $Header: /cvs/krbdev/krb5/src/kadmin/server/server_stubs.c,v 1.40.2.1 2000/06/21 23:47:38 tlyu Exp $
  */
 
-#if !defined(lint) && !defined(__CODECENTER__)
-static char *rcsid = "$Header: /cvs/krbdev/krb5/src/kadmin/server/server_stubs.c,v 1.40.2.1 2000/06/21 23:47:38 tlyu Exp $";
-#endif
-
 #include <gssapi/gssapi.h>
 #include <gssapi/gssapi_krb5.h> /* for gss_nt_krb5_name */
 #include <krb5.h>
@@ -16,6 +11,8 @@ static char *rcsid = "$Header: /cvs/krbdev/krb5/src/kadmin/server/server_stubs.c
 #include <kadm5/server_internal.h>
 #include <kadm5/server_acl.h>
 #include <syslog.h>
+#include <arpa/inet.h>  /* inet_ntoa */
+#include <krb5/adm_proto.h>  /* krb5_klog_syslog */
 #include "misc.h"
 
 #define xdr_free gssrpc_xdr_free /* XXX kludge */
@@ -33,7 +30,15 @@ extern void *				global_server_handle;
 	  cmp_gss_names_rel_1(acceptor_name(rqstp->rq_svccred), \
 			gss_oldchangepw_name)))
 
-int cmp_gss_names(gss_name_t n1, gss_name_t n2)
+
+static int gss_to_krb5_name(kadm5_server_handle_t handle,
+		     gss_name_t gss_name, krb5_principal *princ);
+
+static int gss_name_to_string(gss_name_t gss_name, gss_buffer_desc *str);
+
+static gss_name_t acceptor_name(gss_ctx_id_t context);
+
+static int cmp_gss_names(gss_name_t n1, gss_name_t n2)
 {
    OM_uint32 emaj, emin;
    int equal;
@@ -46,7 +51,7 @@ int cmp_gss_names(gss_name_t n1, gss_name_t n2)
 
 /* Does a comparison of the names and then releases the first entity */
 /* For use above in CHANGEPW_SERVICE */
-int cmp_gss_names_rel_1(gss_name_t n1, gss_name_t n2)
+static int cmp_gss_names_rel_1(gss_name_t n1, gss_name_t n2)
 {
    OM_uint32 min_stat;
    int ret;
@@ -152,6 +157,7 @@ static void free_server_handle(kadm5_server_handle_t handle)
  * server_name, both of which must be freed by the caller.  Returns 0
  * on success and -1 on failure.
  */
+static 
 int setup_gss_names(struct svc_req *rqstp,
 		    gss_buffer_desc *client_name,
 		    gss_buffer_desc *server_name)
@@ -178,7 +184,7 @@ int setup_gss_names(struct svc_req *rqstp,
      return 0;
 }
 
-gss_name_t acceptor_name(gss_ctx_id_t context)
+static gss_name_t acceptor_name(gss_ctx_id_t context)
 {
      OM_uint32 maj_stat, min_stat;
      gss_name_t name;
@@ -190,49 +196,49 @@ gss_name_t acceptor_name(gss_ctx_id_t context)
      return name;
 }
      
-int cmp_gss_krb5_name(kadm5_server_handle_t handle,
+static int cmp_gss_krb5_name(kadm5_server_handle_t handle,
 		      gss_name_t gss_name, krb5_principal princ)
 {
      krb5_principal princ2;
-     int stat;
+     int status;
 
      if (! gss_to_krb5_name(handle, gss_name, &princ2))
 	  return 0;
-     stat = krb5_principal_compare(handle->context, princ, princ2);
+     status = krb5_principal_compare(handle->context, princ, princ2);
      krb5_free_principal(handle->context, princ2);
-     return stat;
+     return status;
 }
 
-int gss_to_krb5_name(kadm5_server_handle_t handle,
+static int gss_to_krb5_name(kadm5_server_handle_t handle,
 		     gss_name_t gss_name, krb5_principal *princ)
 {
-     OM_uint32 stat, minor_stat;
+     OM_uint32 status, minor_stat;
      gss_buffer_desc gss_str;
      gss_OID gss_type;
      int success;
 
-     stat = gss_display_name(&minor_stat, gss_name, &gss_str, &gss_type);
-     if ((stat != GSS_S_COMPLETE) || (gss_type != gss_nt_krb5_name))
+     status = gss_display_name(&minor_stat, gss_name, &gss_str, &gss_type);
+     if ((status != GSS_S_COMPLETE) || (gss_type != gss_nt_krb5_name))
 	  return 0;
      success = (krb5_parse_name(handle->context, gss_str.value, princ) == 0);
      gss_release_buffer(&minor_stat, &gss_str);
      return success;
 }
-int
+
+static int
 gss_name_to_string(gss_name_t gss_name, gss_buffer_desc *str)
 {
-     OM_uint32 stat, minor_stat;
+     OM_uint32 status, minor_stat;
      gss_OID gss_type;
-     int ret;
 
-     stat = gss_display_name(&minor_stat, gss_name, str, &gss_type);
-     if ((stat != GSS_S_COMPLETE) || (gss_type != gss_nt_krb5_name))
+     status = gss_display_name(&minor_stat, gss_name, str, &gss_type);
+     if ((status != GSS_S_COMPLETE) || (gss_type != gss_nt_krb5_name))
 	  return 1;
      return 0;
 }
 
 generic_ret *
-create_principal_1(cprinc_arg *arg, struct svc_req *rqstp)
+create_principal_1_svc(cprinc_arg *arg, struct svc_req *rqstp)
 {
     static generic_ret		ret;
     char			*prime_arg;
@@ -243,10 +249,10 @@ create_principal_1(cprinc_arg *arg, struct svc_req *rqstp)
 
     xdr_free(xdr_generic_ret, &ret);
 
-    if (ret.code = new_server_handle(arg->api_version, rqstp, &handle))
+    if ((ret.code = new_server_handle(arg->api_version, rqstp, &handle)))
 	 return &ret;
 
-    if (ret.code = check_handle((void *)handle)) {
+    if ((ret.code = check_handle((void *)handle))) {
 	 free_server_handle(handle);
 	 return &ret;
     }
@@ -257,7 +263,10 @@ create_principal_1(cprinc_arg *arg, struct svc_req *rqstp)
 	 ret.code = KADM5_FAILURE;
 	 return &ret;
     }
-    krb5_unparse_name(handle->context, arg->rec.principal, &prime_arg);
+    if (krb5_unparse_name(handle->context, arg->rec.principal, &prime_arg)) {
+	 ret.code = KADM5_BAD_PRINCIPAL;
+	 return &ret;
+    }
 
     if (CHANGEPW_SERVICE(rqstp)
 	|| !acl_check(handle->context, rqstp->rq_clntcred, ACL_ADD,
@@ -286,7 +295,7 @@ create_principal_1(cprinc_arg *arg, struct svc_req *rqstp)
 }
 
 generic_ret *
-create_principal3_1(cprinc3_arg *arg, struct svc_req *rqstp)
+create_principal3_1_svc(cprinc3_arg *arg, struct svc_req *rqstp)
 {
     static generic_ret		ret;
     char			*prime_arg;
@@ -297,10 +306,10 @@ create_principal3_1(cprinc3_arg *arg, struct svc_req *rqstp)
 
     xdr_free(xdr_generic_ret, &ret);
 
-    if (ret.code = new_server_handle(arg->api_version, rqstp, &handle))
+    if ((ret.code = new_server_handle(arg->api_version, rqstp, &handle)))
 	 return &ret;
 
-    if (ret.code = check_handle((void *)handle)) {
+    if ((ret.code = check_handle((void *)handle))) {
 	 free_server_handle(handle);
 	 return &ret;
     }
@@ -311,7 +320,10 @@ create_principal3_1(cprinc3_arg *arg, struct svc_req *rqstp)
 	 ret.code = KADM5_FAILURE;
 	 return &ret;
     }
-    krb5_unparse_name(handle->context, arg->rec.principal, &prime_arg);
+    if (krb5_unparse_name(handle->context, arg->rec.principal, &prime_arg)) {
+	 ret.code = KADM5_BAD_PRINCIPAL;
+	 return &ret;
+    }
 
     if (CHANGEPW_SERVICE(rqstp)
 	|| !acl_check(handle->context, rqstp->rq_clntcred, ACL_ADD,
@@ -342,7 +354,7 @@ create_principal3_1(cprinc3_arg *arg, struct svc_req *rqstp)
 }
 
 generic_ret *
-delete_principal_1(dprinc_arg *arg, struct svc_req *rqstp)
+delete_principal_1_svc(dprinc_arg *arg, struct svc_req *rqstp)
 {
     static generic_ret		    ret;
     char			    *prime_arg;
@@ -353,10 +365,10 @@ delete_principal_1(dprinc_arg *arg, struct svc_req *rqstp)
 
     xdr_free(xdr_generic_ret, &ret);
 
-    if (ret.code = new_server_handle(arg->api_version, rqstp, &handle))
+    if ((ret.code = new_server_handle(arg->api_version, rqstp, &handle)))
 	 return &ret;
 
-    if (ret.code = check_handle((void *)handle)) {
+    if ((ret.code = check_handle((void *)handle))) {
 	 free_server_handle(handle);
 	 return &ret;
     }
@@ -367,7 +379,10 @@ delete_principal_1(dprinc_arg *arg, struct svc_req *rqstp)
 	 ret.code = KADM5_FAILURE;
 	 return &ret;
     }
-    krb5_unparse_name(handle->context, arg->princ, &prime_arg);
+    if (krb5_unparse_name(handle->context, arg->princ, &prime_arg)) {
+	 ret.code = KADM5_BAD_PRINCIPAL;
+	 return &ret;
+    }
     
     if (CHANGEPW_SERVICE(rqstp)
 	|| !acl_check(handle->context, rqstp->rq_clntcred, ACL_DELETE,
@@ -391,7 +406,7 @@ delete_principal_1(dprinc_arg *arg, struct svc_req *rqstp)
 }
 
 generic_ret *
-modify_principal_1(mprinc_arg *arg, struct svc_req *rqstp)
+modify_principal_1_svc(mprinc_arg *arg, struct svc_req *rqstp)
 {
     static generic_ret		    ret;
     char			    *prime_arg;
@@ -403,10 +418,10 @@ modify_principal_1(mprinc_arg *arg, struct svc_req *rqstp)
 
     xdr_free(xdr_generic_ret, &ret);
 
-    if (ret.code = new_server_handle(arg->api_version, rqstp, &handle))
+    if ((ret.code = new_server_handle(arg->api_version, rqstp, &handle)))
 	 return &ret;
 
-    if (ret.code = check_handle((void *)handle)) {
+    if ((ret.code = check_handle((void *)handle))) {
 	 free_server_handle(handle);
 	 return &ret;
     }
@@ -415,7 +430,10 @@ modify_principal_1(mprinc_arg *arg, struct svc_req *rqstp)
 	 ret.code = KADM5_FAILURE;
 	 return &ret;
     }
-    krb5_unparse_name(handle->context, arg->rec.principal, &prime_arg);
+    if (krb5_unparse_name(handle->context, arg->rec.principal, &prime_arg)) {
+	 ret.code = KADM5_BAD_PRINCIPAL;
+	 return &ret;
+    }
 
     if (CHANGEPW_SERVICE(rqstp)
 	|| !acl_check(handle->context, rqstp->rq_clntcred, ACL_MODIFY,
@@ -443,7 +461,7 @@ modify_principal_1(mprinc_arg *arg, struct svc_req *rqstp)
 }
 
 generic_ret *
-rename_principal_1(rprinc_arg *arg, struct svc_req *rqstp)
+rename_principal_1_svc(rprinc_arg *arg, struct svc_req *rqstp)
 {
     static generic_ret		ret;
     char			*prime_arg1,
@@ -457,10 +475,10 @@ rename_principal_1(rprinc_arg *arg, struct svc_req *rqstp)
 
     xdr_free(xdr_generic_ret, &ret);
 
-    if (ret.code = new_server_handle(arg->api_version, rqstp, &handle))
+    if ((ret.code = new_server_handle(arg->api_version, rqstp, &handle)))
 	 return &ret;
 
-    if (ret.code = check_handle((void *)handle)) {
+    if ((ret.code = check_handle((void *)handle))) {
 	 free_server_handle(handle);
 	 return &ret;
     }
@@ -469,8 +487,11 @@ rename_principal_1(rprinc_arg *arg, struct svc_req *rqstp)
 	 ret.code = KADM5_FAILURE;
 	 return &ret;
     }
-    krb5_unparse_name(handle->context, arg->src, &prime_arg1);
-    krb5_unparse_name(handle->context, arg->dest, &prime_arg2);
+    if (krb5_unparse_name(handle->context, arg->src, &prime_arg1) ||
+        krb5_unparse_name(handle->context, arg->dest, &prime_arg2)) {
+	 ret.code = KADM5_BAD_PRINCIPAL;
+	 return &ret;
+    }
     sprintf(prime_arg, "%s to %s", prime_arg1, prime_arg2);
 
     ret.code = KADM5_OK;
@@ -510,7 +531,7 @@ rename_principal_1(rprinc_arg *arg, struct svc_req *rqstp)
 }
 
 gprinc_ret *
-get_principal_1(gprinc_arg *arg, struct svc_req *rqstp)
+get_principal_1_svc(gprinc_arg *arg, struct svc_req *rqstp)
 {
     static gprinc_ret		    ret;
     kadm5_principal_ent_t_v1	    e;
@@ -522,10 +543,10 @@ get_principal_1(gprinc_arg *arg, struct svc_req *rqstp)
 
     xdr_free(xdr_gprinc_ret, &ret);
 
-    if (ret.code = new_server_handle(arg->api_version, rqstp, &handle))
+    if ((ret.code = new_server_handle(arg->api_version, rqstp, &handle)))
 	 return &ret;
 
-    if (ret.code = check_handle((void *)handle)) {
+    if ((ret.code = check_handle((void *)handle))) {
 	 free_server_handle(handle);
 	 return &ret;
     }
@@ -539,7 +560,10 @@ get_principal_1(gprinc_arg *arg, struct svc_req *rqstp)
 	 ret.code = KADM5_FAILURE;
 	 return &ret;
     }
-    krb5_unparse_name(handle->context, arg->princ, &prime_arg);
+    if (krb5_unparse_name(handle->context, arg->princ, &prime_arg)) {
+	 ret.code = KADM5_BAD_PRINCIPAL;
+	 return &ret;
+    }
 
     if (! cmp_gss_krb5_name(handle, rqstp->rq_clntcred, arg->princ) &&
 	(CHANGEPW_SERVICE(rqstp) || !acl_check(handle->context,
@@ -579,7 +603,7 @@ get_principal_1(gprinc_arg *arg, struct svc_req *rqstp)
 }
 
 gprincs_ret *
-get_princs_1(gprincs_arg *arg, struct svc_req *rqstp)
+get_princs_1_svc(gprincs_arg *arg, struct svc_req *rqstp)
 {
     static gprincs_ret		    ret;
     char			    *prime_arg;
@@ -590,10 +614,10 @@ get_princs_1(gprincs_arg *arg, struct svc_req *rqstp)
 
     xdr_free(xdr_gprincs_ret, &ret);
 
-    if (ret.code = new_server_handle(arg->api_version, rqstp, &handle))
+    if ((ret.code = new_server_handle(arg->api_version, rqstp, &handle)))
 	 return &ret;
 
-    if (ret.code = check_handle((void *)handle)) {
+    if ((ret.code = check_handle((void *)handle))) {
 	 free_server_handle(handle);
 	 return &ret;
     }
@@ -634,7 +658,7 @@ get_princs_1(gprincs_arg *arg, struct svc_req *rqstp)
 }
 
 generic_ret *
-chpass_principal_1(chpass_arg *arg, struct svc_req *rqstp)
+chpass_principal_1_svc(chpass_arg *arg, struct svc_req *rqstp)
 {
     static generic_ret		    ret;
     char			    *prime_arg;
@@ -645,10 +669,10 @@ chpass_principal_1(chpass_arg *arg, struct svc_req *rqstp)
 
     xdr_free(xdr_generic_ret, &ret);
 
-    if (ret.code = new_server_handle(arg->api_version, rqstp, &handle))
+    if ((ret.code = new_server_handle(arg->api_version, rqstp, &handle)))
 	 return &ret;
 
-    if (ret.code = check_handle((void *)handle)) {
+    if ((ret.code = check_handle((void *)handle))) {
 	 free_server_handle(handle);
 	 return &ret;
     }
@@ -659,11 +683,14 @@ chpass_principal_1(chpass_arg *arg, struct svc_req *rqstp)
 	 ret.code = KADM5_FAILURE;
 	 return &ret;
     }
-    krb5_unparse_name(handle->context, arg->princ, &prime_arg);
+    if (krb5_unparse_name(handle->context, arg->princ, &prime_arg)) {
+	 ret.code = KADM5_BAD_PRINCIPAL;
+	 return &ret;
+    }
 
     if (cmp_gss_krb5_name(handle, rqstp->rq_clntcred, arg->princ)) {
-	 ret.code = chpass_principal_wrapper((void *)handle, arg->princ,
-					     arg->pass);
+	 ret.code = chpass_principal_wrapper_3((void *)handle, arg->princ,
+					       FALSE, 0, NULL, arg->pass);
     } else if (!(CHANGEPW_SERVICE(rqstp)) &&
 	       acl_check(handle->context, rqstp->rq_clntcred,
 			 ACL_CHANGEPW, arg->princ, NULL)) {
@@ -692,7 +719,7 @@ chpass_principal_1(chpass_arg *arg, struct svc_req *rqstp)
 }
 
 generic_ret *
-chpass_principal3_1(chpass3_arg *arg, struct svc_req *rqstp)
+chpass_principal3_1_svc(chpass3_arg *arg, struct svc_req *rqstp)
 {
     static generic_ret		    ret;
     char			    *prime_arg;
@@ -703,10 +730,10 @@ chpass_principal3_1(chpass3_arg *arg, struct svc_req *rqstp)
 
     xdr_free(xdr_generic_ret, &ret);
 
-    if (ret.code = new_server_handle(arg->api_version, rqstp, &handle))
+    if ((ret.code = new_server_handle(arg->api_version, rqstp, &handle)))
 	 return &ret;
 
-    if (ret.code = check_handle((void *)handle)) {
+    if ((ret.code = check_handle((void *)handle))) {
 	 free_server_handle(handle);
 	 return &ret;
     }
@@ -717,11 +744,17 @@ chpass_principal3_1(chpass3_arg *arg, struct svc_req *rqstp)
 	 ret.code = KADM5_FAILURE;
 	 return &ret;
     }
-    krb5_unparse_name(handle->context, arg->princ, &prime_arg);
+    if (krb5_unparse_name(handle->context, arg->princ, &prime_arg)) {
+	 ret.code = KADM5_BAD_PRINCIPAL;
+	 return &ret;
+    }
 
     if (cmp_gss_krb5_name(handle, rqstp->rq_clntcred, arg->princ)) {
-	 ret.code = chpass_principal_wrapper((void *)handle, arg->princ,
-					     arg->pass);
+	 ret.code = chpass_principal_wrapper_3((void *)handle, arg->princ,
+					       arg->keepold,
+					       arg->n_ks_tuple,
+					       arg->ks_tuple,
+					       arg->pass);
     } else if (!(CHANGEPW_SERVICE(rqstp)) &&
 	       acl_check(handle->context, rqstp->rq_clntcred,
 			 ACL_CHANGEPW, arg->princ, NULL)) {
@@ -753,7 +786,7 @@ chpass_principal3_1(chpass3_arg *arg, struct svc_req *rqstp)
 }
 
 generic_ret *
-setv4key_principal_1(setv4key_arg *arg, struct svc_req *rqstp)
+setv4key_principal_1_svc(setv4key_arg *arg, struct svc_req *rqstp)
 {
     static generic_ret		    ret;
     char			    *prime_arg;
@@ -764,10 +797,10 @@ setv4key_principal_1(setv4key_arg *arg, struct svc_req *rqstp)
 
     xdr_free(xdr_generic_ret, &ret);
 
-    if (ret.code = new_server_handle(arg->api_version, rqstp, &handle))
+    if ((ret.code = new_server_handle(arg->api_version, rqstp, &handle)))
 	 return &ret;
 
-    if (ret.code = check_handle((void *)handle)) {
+    if ((ret.code = check_handle((void *)handle))) {
 	 free_server_handle(handle);
 	 return &ret;
     }
@@ -778,7 +811,10 @@ setv4key_principal_1(setv4key_arg *arg, struct svc_req *rqstp)
 	 ret.code = KADM5_FAILURE;
 	 return &ret;
     }
-    krb5_unparse_name(handle->context, arg->princ, &prime_arg);
+    if (krb5_unparse_name(handle->context, arg->princ, &prime_arg)) {
+	 ret.code = KADM5_BAD_PRINCIPAL;
+	 return &ret;
+    }
 
     if (!(CHANGEPW_SERVICE(rqstp)) &&
 	       acl_check(handle->context, rqstp->rq_clntcred,
@@ -808,7 +844,7 @@ setv4key_principal_1(setv4key_arg *arg, struct svc_req *rqstp)
 }
 
 generic_ret *
-setkey_principal_1(setkey_arg *arg, struct svc_req *rqstp)
+setkey_principal_1_svc(setkey_arg *arg, struct svc_req *rqstp)
 {
     static generic_ret		    ret;
     char			    *prime_arg;
@@ -819,10 +855,10 @@ setkey_principal_1(setkey_arg *arg, struct svc_req *rqstp)
 
     xdr_free(xdr_generic_ret, &ret);
 
-    if (ret.code = new_server_handle(arg->api_version, rqstp, &handle))
+    if ((ret.code = new_server_handle(arg->api_version, rqstp, &handle)))
 	 return &ret;
 
-    if (ret.code = check_handle((void *)handle)) {
+    if ((ret.code = check_handle((void *)handle))) {
 	 free_server_handle(handle);
 	 return &ret;
     }
@@ -833,7 +869,10 @@ setkey_principal_1(setkey_arg *arg, struct svc_req *rqstp)
 	 ret.code = KADM5_FAILURE;
 	 return &ret;
     }
-    krb5_unparse_name(handle->context, arg->princ, &prime_arg);
+    if (krb5_unparse_name(handle->context, arg->princ, &prime_arg)) {
+	 ret.code = KADM5_BAD_PRINCIPAL;
+	 return &ret;
+    }
 
     if (!(CHANGEPW_SERVICE(rqstp)) &&
 	       acl_check(handle->context, rqstp->rq_clntcred,
@@ -863,7 +902,7 @@ setkey_principal_1(setkey_arg *arg, struct svc_req *rqstp)
 }
 
 generic_ret *
-setkey_principal3_1(setkey3_arg *arg, struct svc_req *rqstp)
+setkey_principal3_1_svc(setkey3_arg *arg, struct svc_req *rqstp)
 {
     static generic_ret		    ret;
     char			    *prime_arg;
@@ -874,10 +913,10 @@ setkey_principal3_1(setkey3_arg *arg, struct svc_req *rqstp)
 
     xdr_free(xdr_generic_ret, &ret);
 
-    if (ret.code = new_server_handle(arg->api_version, rqstp, &handle))
+    if ((ret.code = new_server_handle(arg->api_version, rqstp, &handle)))
 	 return &ret;
 
-    if (ret.code = check_handle((void *)handle)) {
+    if ((ret.code = check_handle((void *)handle))) {
 	 free_server_handle(handle);
 	 return &ret;
     }
@@ -888,7 +927,10 @@ setkey_principal3_1(setkey3_arg *arg, struct svc_req *rqstp)
 	 ret.code = KADM5_FAILURE;
 	 return &ret;
     }
-    krb5_unparse_name(handle->context, arg->princ, &prime_arg);
+    if (krb5_unparse_name(handle->context, arg->princ, &prime_arg)) {
+	 ret.code = KADM5_BAD_PRINCIPAL;
+	 return &ret;
+    }
 
     if (!(CHANGEPW_SERVICE(rqstp)) &&
 	       acl_check(handle->context, rqstp->rq_clntcred,
@@ -921,7 +963,7 @@ setkey_principal3_1(setkey3_arg *arg, struct svc_req *rqstp)
 }
 
 chrand_ret *
-chrand_principal_1(chrand_arg *arg, struct svc_req *rqstp)
+chrand_principal_1_svc(chrand_arg *arg, struct svc_req *rqstp)
 {
     static chrand_ret		ret;
     krb5_keyblock		*k;
@@ -934,10 +976,10 @@ chrand_principal_1(chrand_arg *arg, struct svc_req *rqstp)
 
     xdr_free(xdr_chrand_ret, &ret);
 
-    if (ret.code = new_server_handle(arg->api_version, rqstp, &handle))
+    if ((ret.code = new_server_handle(arg->api_version, rqstp, &handle)))
 	 return &ret;
 
-    if (ret.code = check_handle((void *)handle)) {
+    if ((ret.code = check_handle((void *)handle))) {
 	 free_server_handle(handle);
 	 return &ret;
     }
@@ -952,11 +994,14 @@ chrand_principal_1(chrand_arg *arg, struct svc_req *rqstp)
 	 free_server_handle(handle);
 	 return &ret;
     }
-    krb5_unparse_name(handle->context, arg->princ, &prime_arg);
+    if (krb5_unparse_name(handle->context, arg->princ, &prime_arg)) {
+	 ret.code = KADM5_BAD_PRINCIPAL;
+	 return &ret;
+    }
 
     if (cmp_gss_krb5_name(handle, rqstp->rq_clntcred, arg->princ)) {
-	 ret.code = randkey_principal_wrapper((void *)handle,
-					      arg->princ, &k, &nkeys); 
+	 ret.code = randkey_principal_wrapper_3((void *)handle, arg->princ,
+						FALSE, 0, NULL, &k, &nkeys);
     } else if (!(CHANGEPW_SERVICE(rqstp)) &&
 	       acl_check(handle->context, rqstp->rq_clntcred,
 			 ACL_CHANGEPW, arg->princ, NULL)) {
@@ -994,7 +1039,7 @@ chrand_principal_1(chrand_arg *arg, struct svc_req *rqstp)
 }
 
 chrand_ret *
-chrand_principal3_1(chrand3_arg *arg, struct svc_req *rqstp)
+chrand_principal3_1_svc(chrand3_arg *arg, struct svc_req *rqstp)
 {
     static chrand_ret		ret;
     krb5_keyblock		*k;
@@ -1007,10 +1052,10 @@ chrand_principal3_1(chrand3_arg *arg, struct svc_req *rqstp)
 
     xdr_free(xdr_chrand_ret, &ret);
 
-    if (ret.code = new_server_handle(arg->api_version, rqstp, &handle))
+    if ((ret.code = new_server_handle(arg->api_version, rqstp, &handle)))
 	 return &ret;
 
-    if (ret.code = check_handle((void *)handle)) {
+    if ((ret.code = check_handle((void *)handle))) {
 	 free_server_handle(handle);
 	 return &ret;
     }
@@ -1025,11 +1070,17 @@ chrand_principal3_1(chrand3_arg *arg, struct svc_req *rqstp)
 	 free_server_handle(handle);
 	 return &ret;
     }
-    krb5_unparse_name(handle->context, arg->princ, &prime_arg);
+    if (krb5_unparse_name(handle->context, arg->princ, &prime_arg)) {
+	 ret.code = KADM5_BAD_PRINCIPAL;
+	 return &ret;
+    }
 
     if (cmp_gss_krb5_name(handle, rqstp->rq_clntcred, arg->princ)) {
-	 ret.code = randkey_principal_wrapper((void *)handle,
-					      arg->princ, &k, &nkeys); 
+	 ret.code = randkey_principal_wrapper_3((void *)handle, arg->princ,
+						arg->keepold,
+						arg->n_ks_tuple,
+						arg->ks_tuple,
+						&k, &nkeys);
     } else if (!(CHANGEPW_SERVICE(rqstp)) &&
 	       acl_check(handle->context, rqstp->rq_clntcred,
 			 ACL_CHANGEPW, arg->princ, NULL)) {
@@ -1070,7 +1121,7 @@ chrand_principal3_1(chrand3_arg *arg, struct svc_req *rqstp)
 }
 
 generic_ret *
-create_policy_1(cpol_arg *arg, struct svc_req *rqstp)
+create_policy_1_svc(cpol_arg *arg, struct svc_req *rqstp)
 {
     static generic_ret		    ret;
     char			    *prime_arg;
@@ -1081,10 +1132,10 @@ create_policy_1(cpol_arg *arg, struct svc_req *rqstp)
 
     xdr_free(xdr_generic_ret, &ret);
 
-    if (ret.code = new_server_handle(arg->api_version, rqstp, &handle))
+    if ((ret.code = new_server_handle(arg->api_version, rqstp, &handle)))
 	 return &ret;
 
-    if (ret.code = check_handle((void *)handle)) {
+    if ((ret.code = check_handle((void *)handle))) {
 	 free_server_handle(handle);
 	 return &ret;
     }
@@ -1121,7 +1172,7 @@ create_policy_1(cpol_arg *arg, struct svc_req *rqstp)
 }
 
 generic_ret *
-delete_policy_1(dpol_arg *arg, struct svc_req *rqstp)
+delete_policy_1_svc(dpol_arg *arg, struct svc_req *rqstp)
 {
     static generic_ret		    ret;
     char			    *prime_arg;
@@ -1132,10 +1183,10 @@ delete_policy_1(dpol_arg *arg, struct svc_req *rqstp)
 
     xdr_free(xdr_generic_ret, &ret);
 
-    if (ret.code = new_server_handle(arg->api_version, rqstp, &handle))
+    if ((ret.code = new_server_handle(arg->api_version, rqstp, &handle)))
 	 return &ret;
 
-    if (ret.code = check_handle((void *)handle)) {
+    if ((ret.code = check_handle((void *)handle))) {
 	 free_server_handle(handle);
 	 return &ret;
     }
@@ -1170,7 +1221,7 @@ delete_policy_1(dpol_arg *arg, struct svc_req *rqstp)
 }
 
 generic_ret *
-modify_policy_1(mpol_arg *arg, struct svc_req *rqstp)
+modify_policy_1_svc(mpol_arg *arg, struct svc_req *rqstp)
 {
     static generic_ret		    ret;
     char			    *prime_arg;
@@ -1181,10 +1232,10 @@ modify_policy_1(mpol_arg *arg, struct svc_req *rqstp)
 
     xdr_free(xdr_generic_ret, &ret);
 
-    if (ret.code = new_server_handle(arg->api_version, rqstp, &handle))
+    if ((ret.code = new_server_handle(arg->api_version, rqstp, &handle)))
 	 return &ret;
 
-    if (ret.code = check_handle((void *)handle)) {
+    if ((ret.code = check_handle((void *)handle))) {
 	 free_server_handle(handle);
 	 return &ret;
     }
@@ -1220,7 +1271,7 @@ modify_policy_1(mpol_arg *arg, struct svc_req *rqstp)
 }
 
 gpol_ret * 
-get_policy_1(gpol_arg *arg, struct svc_req *rqstp)
+get_policy_1_svc(gpol_arg *arg, struct svc_req *rqstp)
 {
     static gpol_ret		ret;
     kadm5_ret_t		ret2;
@@ -1230,15 +1281,14 @@ get_policy_1(gpol_arg *arg, struct svc_req *rqstp)
     OM_uint32			minor_stat;    
     kadm5_policy_ent_t	e;
     kadm5_principal_ent_rec	caller_ent;
-    krb5_principal		caller;
     kadm5_server_handle_t	handle;
 
     xdr_free(xdr_gpol_ret,  &ret);
 
-    if (ret.code = new_server_handle(arg->api_version, rqstp, &handle))
+    if ((ret.code = new_server_handle(arg->api_version, rqstp, &handle)))
 	 return &ret;
 
-    if (ret.code = check_handle((void *)handle)) {
+    if ((ret.code = check_handle((void *)handle))) {
 	 free_server_handle(handle);
 	 return &ret;
     }
@@ -1305,7 +1355,7 @@ get_policy_1(gpol_arg *arg, struct svc_req *rqstp)
 }
 
 gpols_ret *
-get_pols_1(gpols_arg *arg, struct svc_req *rqstp)
+get_pols_1_svc(gpols_arg *arg, struct svc_req *rqstp)
 {
     static gpols_ret		    ret;
     char			    *prime_arg;
@@ -1316,10 +1366,10 @@ get_pols_1(gpols_arg *arg, struct svc_req *rqstp)
 
     xdr_free(xdr_gpols_ret, &ret);
 
-    if (ret.code = new_server_handle(arg->api_version, rqstp, &handle))
+    if ((ret.code = new_server_handle(arg->api_version, rqstp, &handle)))
 	 return &ret;
 
-    if (ret.code = check_handle((void *)handle)) {
+    if ((ret.code = check_handle((void *)handle))) {
 	 free_server_handle(handle);
 	 return &ret;
     }
@@ -1357,20 +1407,19 @@ get_pols_1(gpols_arg *arg, struct svc_req *rqstp)
     return &ret;
 }
 
-getprivs_ret * get_privs_1(krb5_ui_4 *arg, struct svc_req *rqstp)
+getprivs_ret * get_privs_1_svc(krb5_ui_4 *arg, struct svc_req *rqstp)
 {
      static getprivs_ret	    ret;
-     char			    *prime_arg;
      gss_buffer_desc		    client_name, service_name;
      OM_uint32			    minor_stat;
      kadm5_server_handle_t	    handle;
 
      xdr_free(xdr_getprivs_ret, &ret);
 
-     if (ret.code = new_server_handle(*arg, rqstp, &handle))
+     if ((ret.code = new_server_handle(*arg, rqstp, &handle)))
 	  return &ret;
 
-     if (ret.code = check_handle((void *)handle)) {
+     if ((ret.code = check_handle((void *)handle))) {
 	  free_server_handle(handle);
 	  return &ret;
      }
@@ -1394,7 +1443,7 @@ getprivs_ret * get_privs_1(krb5_ui_4 *arg, struct svc_req *rqstp)
      return &ret;
 }
 
-generic_ret *init_1(krb5_ui_4 *arg, struct svc_req *rqstp)
+generic_ret *init_1_svc(krb5_ui_4 *arg, struct svc_req *rqstp)
 {
      static generic_ret		ret;
      gss_buffer_desc		client_name,
@@ -1404,7 +1453,7 @@ generic_ret *init_1(krb5_ui_4 *arg, struct svc_req *rqstp)
 
      xdr_free(xdr_generic_ret, &ret);
 
-     if (ret.code = new_server_handle(*arg, rqstp, &handle))
+     if ((ret.code = new_server_handle(*arg, rqstp, &handle)))
 	  return &ret;
      if (! (ret.code = check_handle((void *)handle))) {
 	 ret.api_version = handle->api_version;
