@@ -71,13 +71,20 @@ extern Kadm_Server server_parm;
 krb5_context kadm_context;
 int debug;
 
+static void cleanexit(int);
+static int kadm_listen(void);
+static void process_client(int, struct sockaddr_in *);
+static void kill_children(void);
+static krb5_sigtype do_child(int);
+
+
 /* close the system log file */
-void close_syslog()
+static void close_syslog()
 {
    syslog(LOG_INFO, "Shutting down V4 admin server");
 }
 
-void byebye()			/* say goodnight gracie */
+static void byebye()			/* say goodnight gracie */
 {
    printf("Admin Server (kadm server) has completed operation.\n");
 }
@@ -86,6 +93,7 @@ void byebye()			/* say goodnight gracie */
 ** Main does the logical thing, it sets up the database and RPC interface,
 **  as well as handling the creation and maintenance of the syslog file...
 */
+int
 main(argc, argv)		/* admin_server main routine */
 int argc;
 char *argv[];
@@ -168,7 +176,8 @@ char *argv[];
 	}
 
     if (krbrlm[0] == 0) {
-	if (errval = krb5_get_default_realm(kadm_context, &lrealm)) {
+	errval = krb5_get_default_realm(kadm_context, &lrealm);
+	if (errval) {
 	    com_err(argv[0], errval, "while attempting to get local realm");
 	    exit(1);
 	}
@@ -179,12 +188,15 @@ char *argv[];
     paramsin.realm = krbrlm;
     paramsin.mask |= KADM5_CONFIG_REALM;
 
-    if (errval = kadm5_get_config_params(kadm_context, NULL, NULL,
-					 &paramsin, &paramsout)) {
+    errval = kadm5_get_config_params(kadm_context, NULL, NULL,
+				     &paramsin, &paramsout);
+    if (errval) {
 	 com_err(argv[0], errval, "while retrieving kadm5 params");
 	 exit(1);
     }
-    if (errval = krb5_db_set_name(kadm_context, paramsout.dbname)) {
+
+    errval = krb5_db_set_name(kadm_context, paramsout.dbname);
+    if (errval) {
 	 com_err(argv[0], errval, "while setting dbname");
 	 exit(1);
     }
@@ -195,8 +207,8 @@ char *argv[];
     printf("regular kill instead\n\n");
 
 #ifdef KADM5
-    printf("KADM Server starting in the KADM5 mode (%sprocess id %d).\n",
-	   debug ? "" : "parent ", getpid());
+    printf("KADM Server starting in the KADM5 mode (%sprocess id %ld).\n",
+	   debug ? "" : "parent ", (long) getpid());
 #else
     printf("KADM Server starting in %s mode for the purposes for password changing\n\n", fascist_cpw ? "fascist" : "NON-FASCIST");
 #endif
@@ -211,7 +223,9 @@ char *argv[];
 	byebye();
 	exit(1);
     }
-    if (errval = krb5_db_set_lockmode(kadm_context, TRUE)) {
+
+    errval = krb5_db_set_lockmode(kadm_context, TRUE);
+    if (errval) {
 	com_err(argv[0], errval, "while setting db to nonblocking");
 	close_syslog();
 	krb5_db_fini(kadm_context);
@@ -249,9 +263,9 @@ static void clear_secrets()
     return;
 }
 
-static exit_now = 0;
+static int exit_now = 0;
 
-krb5_sigtype
+static krb5_sigtype
 doexit(sig)
 	int sig;
 {
@@ -267,22 +281,21 @@ int unknown_child = 0;
 kadm_listen
 listen on the admin servers port for a request
 */
+static int
 kadm_listen()
 {
-    extern int errno;
     int found;
     int admin_fd;
     int peer_fd;
     fd_set mask, readfds;
     struct sockaddr_in peer;
-    int addrlen;
-    void process_client(), kill_children();
+    socklen_t addrlen;
     int pid;
-    krb5_sigtype do_child();
 #ifdef POSIX_SIGNALS
     struct sigaction new_act;
 
     new_act.sa_handler = doexit;
+    new_act.sa_flags = 0;
     sigemptyset(&new_act.sa_mask);
     sigaction(SIGINT, &new_act, 0);
     sigaction(SIGTERM, &new_act, 0);
@@ -348,7 +361,7 @@ kadm_listen()
 
 	    if (debug) {
 		 process_client(peer_fd, &peer);
-	    } else if (pid = fork()) {
+	    } else if ((pid = fork())) {
 		/* parent */
 		if (pid < 0) {
 		    syslog(LOG_ERR, "fork: %s", error_message(errno));
@@ -380,7 +393,7 @@ kadm_listen()
     /*NOTREACHED*/
 }
 
-void process_client(fd, who)
+static void process_client(fd, who)
    int fd;
    struct sockaddr_in *who;
 {
@@ -421,7 +434,9 @@ void process_client(fd, who)
 	 cleanexit(1);
     }
     free(service_name);
-    if (retval = krb5_db_set_name(kadm_context, paramsout.dbname)) {
+
+    retval = krb5_db_set_name(kadm_context, paramsout.dbname);
+    if (retval) {
 	 syslog(LOG_ERR, "%s while setting dbname", error_message(retval));
 	 cleanexit(1);
     }
@@ -438,7 +453,8 @@ void process_client(fd, who)
 
     server_parm.recv_addr = *who;
 
-    if (retval = krb5_db_init(kadm_context)) {	/* Open as client */
+    retval = krb5_db_init(kadm_context);
+    if (retval) {	/* Open as client */
 	syslog(LOG_ERR, "can't open krb db: %s", error_message(retval));
 	cleanexit(1);
     }
@@ -450,7 +466,7 @@ void process_client(fd, who)
     /* ugh... clean this up later */
     if (status == KRB5_KDB_DB_INUSE) {
 	/* db locked */
-	krb5_ui_4 retcode = KADM_DB_INUSE;
+	krb5_ui_4 retcode;
 	char *pdat;
 	
 	dat_len = KADM_VERSIZE + sizeof(krb5_ui_4);
@@ -555,7 +571,7 @@ void process_client(fd, who)
     /*NOTREACHED*/
 }
 
-krb5_sigtype
+static krb5_sigtype
 do_child(sig)
 	int sig;
 {
@@ -605,14 +621,16 @@ do_child(sig)
     SIGNAL_RETURN;
 }
 
-cleanexit(val)
+static
+void cleanexit(val)
+    int val;
 {
     krb5_db_fini(kadm_context);
     clear_secrets();
     exit(val);
 }
 
-void
+static void
 kill_children()
 {
     register int i;
@@ -643,8 +661,8 @@ kill_children()
 }
 
 #ifdef KADM5
-krb5_ui_4 convert_kadm5_to_kadm(val)
-   krb5_ui_4 val;
+krb5_int32 convert_kadm5_to_kadm(val)
+   krb5_int32 val;
 {
      switch (val) {
      case KADM5_AUTH_GET:
