@@ -16,7 +16,10 @@
  * this permission notice appear in supporting documentation, and that
  * the name of M.I.T. not be used in advertising or publicity pertaining
  * to distribution of the software without specific, written prior
- * permission.  M.I.T. makes no representations about the suitability of
+ * permission.  Furthermore if you modify this software you must label
+ * your software as modified software and not distribute it in such a
+ * fashion that it might be confused with the original M.I.T. software.
+ * M.I.T. makes no representations about the suitability of
  * this software for any purpose.  It is provided "as is" without express
  * or implied warranty.
  *
@@ -40,36 +43,36 @@
 /*
  * Strings
  */
-static char *kadm_cache_name_fmt =	"FILE:/tmp/tkt_kadm_%d";
+#define kadm_cache_name_fmt	"FILE:/tmp/tkt_kadm_%d"
 
 /*
  * Prototypes for local functions
  */
 static krb5_error_code kadm_get_ccache
-	PROTOTYPE((krb5_context,
+	(krb5_context,
 		   char *,
 		   char *,
 		   krb5_ccache *,
-		   krb5_principal *));
+		   krb5_principal *);
 static krb5_error_code kadm_get_creds
-	PROTOTYPE((krb5_context,
+	(krb5_context,
 		krb5_ccache ,
 		krb5_principal,
 		krb5_creds  *,
+		const char *,
 		char *,
-		char *,
-		krb5_timestamp));
+		krb5_timestamp);
 static krb5_error_code kadm_contact_server
-	PROTOTYPE((krb5_context,
+	(krb5_context,
 		krb5_data *,
 		int *,
 		krb5_address **,
-		krb5_address **));
+		krb5_address **);
 static krb5_error_code kadm_get_auth
-	PROTOTYPE((krb5_context,
+	(krb5_context,
 		krb5_auth_context *,
 		krb5_address *,
-		krb5_address *));
+		krb5_address *);
 
 /*
  * kadm_get_ccache()	- Initialze a credentials cache.
@@ -123,20 +126,21 @@ kadm_get_ccache(kcontext, user, ccname, ccache, client)
     }
 
     /* Parse the name and form our principal */
-    if (kret = krb5_parse_name(kcontext, name, client))
+    kret = krb5_parse_name(kcontext, name, client);
+    if (kret)
 	goto cleanup;
 
     if (!ccname) {
-#if defined(_MSDOS) || defined(_WIN32)
+#if defined(_WIN32)
 	strcpy (new_cache, "FILE:");
 	GetTempFileName (0, "tkt", 0, new_cache+5);
 #else
 #ifdef _MACINTOSH
 	(void) sprintf(new_cache, "STDIO:admcc");
 #else
-	(void) sprintf(new_cache, kadm_cache_name_fmt, getpid());
+	(void) sprintf(new_cache, kadm_cache_name_fmt, (int) getpid());
 #endif /* _MACINTOSH */
-#endif /* _MSDOS || _WIN32 */
+#endif /* _WIN32 */
     }
     else
 	sprintf(new_cache, "FILE:%s", ccname);
@@ -181,14 +185,14 @@ kadm_get_creds(kcontext, ccache, client, creds, prompt, oldpw, tlife)
     krb5_ccache		ccache;
     krb5_principal	client;
     krb5_creds		*creds;
-    char		*prompt;
+    const char		*prompt;
     char		*oldpw;
     krb5_timestamp	tlife;
 {
     char		*client_name;
     krb5_error_code	kret;
     krb5_address	**my_addresses;
-    int			old_pwsize;
+    unsigned int	old_pwsize;
     krb5_creds		tcreds;
 
     /* Initialize */
@@ -196,10 +200,12 @@ kadm_get_creds(kcontext, ccache, client, creds, prompt, oldpw, tlife)
     client_name = (char *) NULL;
 
     /* Get the string form for our principal */
-    if (kret = krb5_unparse_name(kcontext, client, &client_name))
+    kret = krb5_unparse_name(kcontext, client, &client_name);
+    if (kret)
 	return(kret);
 
-    if (kret = krb5_os_localaddr(kcontext, &my_addresses))
+    kret = krb5_os_localaddr(kcontext, &my_addresses);
+    if (kret)
 	goto cleanup;
 
     creds->client = client;
@@ -209,15 +215,16 @@ kadm_get_creds(kcontext, ccache, client, creds, prompt, oldpw, tlife)
      *	realm name is instance
      *	realm name is realm name
      */
-    if (kret = krb5_build_principal_ext(kcontext,
-					&creds->server,
-					client->realm.length,
-					client->realm.data,
-					strlen(KRB5_ADM_SERVICE_INSTANCE),
-					KRB5_ADM_SERVICE_INSTANCE,
-					client->realm.length,
-					client->realm.data,
-					0))
+    kret = krb5_build_principal_ext(kcontext,
+				    &creds->server,
+				    client->realm.length,
+				    client->realm.data,
+				    strlen(KRB5_ADM_SERVICE_INSTANCE),
+				    KRB5_ADM_SERVICE_INSTANCE,
+				    client->realm.length,
+				    client->realm.data,
+				    0);
+    if (kret)
 	goto cleanup;
 
     /* Attempt to retrieve an appropriate entry from the credentials cache. */
@@ -232,15 +239,16 @@ kadm_get_creds(kcontext, ccache, client, creds, prompt, oldpw, tlife)
 	if (prompt != (char *) NULL) {
 	    /* Read the password */
 	    old_pwsize = KRB5_ADM_MAX_PASSWORD_LEN;
-	    if (kret = krb5_read_password(kcontext,
-					  prompt,
-					  (char *) NULL,
-					  oldpw,
-					  &old_pwsize))
+	    kret = krb5_read_password(kcontext, prompt, (char *) NULL,
+					  oldpw, &old_pwsize);
+	    if (kret)
 		goto cleanup;
 	}
-	if (kret = krb5_timeofday(kcontext, &jetzt))
+
+	kret = krb5_timeofday(kcontext, &jetzt);
+	if (kret)
 	    goto cleanup;
+
 	if (tlife > 0)
 	    creds->times.endtime = jetzt + tlife;
 	else
@@ -318,7 +326,7 @@ kadm_contact_server(kcontext, realmp, sockp, local, remote)
 
     struct sockaddr_in	in_local;
     struct sockaddr_in	in_remote;
-    int			addr_len;
+    socklen_t		addr_len;
 
     const char		*realm_admin_names[4];
     char		*realm_name;
@@ -332,7 +340,7 @@ kadm_contact_server(kcontext, realmp, sockp, local, remote)
     /*
      * XXX - only know ADDRTYPE_INET.
      */
-#ifdef	KRB5_USE_INET
+#ifdef	HAVE_NETINET_IN_H
     *local = (krb5_address *) malloc(sizeof(krb5_address));
     *remote = (krb5_address *) malloc(sizeof(krb5_address));
     realm_name = (char *) malloc((size_t) realmp->length + 1);
@@ -477,7 +485,8 @@ kadm_contact_server(kcontext, realmp, sockp, local, remote)
 	}
 	in_remote.sin_port = service->s_port;
 	
-	if (kret = krb5_get_krbhst(kcontext, realmp, &hostlist))
+	kret = krb5_get_krbhst(kcontext, realmp, &hostlist);
+	if (kret)
 	    goto cleanup;
 	
 	/* Now count the number of hosts in the realm */
@@ -539,9 +548,9 @@ kadm_contact_server(kcontext, realmp, sockp, local, remote)
 	if (!found)
 	    kret = KRB5_SERVICE_UNKNOWN;
     }
-#else	/* KRB5_USE_INET */
+#else	/* HAVE_NETINET_IN_H */
     kret = ENOENT;
-#endif	/* KRB5_USE_INET */
+#endif	/* HAVE_NETINET_IN_H */
 
  cleanup:
     if (kret) {
@@ -602,12 +611,12 @@ kadm_get_auth(kcontext, ctxp, local, remote)
  *	free auth_context
  *	close socket.
  */
-krb5_error_code INTERFACE
+krb5_error_code KRB5_CALLCONV
 krb5_adm_connect(kcontext, user, prompt, opassword, sockp, ctxp,
 		 ccachep, ccname, tlife)
     krb5_context	kcontext;	/* Context handle	(In ) */
     char		*user;		/* User specified	(In ) */
-    char		*prompt;	/* Old password prompt	(In ) */
+    const char		*prompt;	/* Old password prompt	(In ) */
     char		*opassword;	/* Old Password		(I/O) */
     int			*sockp;		/* Socket for conn.	(Out) */
     krb5_auth_context	*ctxp;		/* Auth context		(Out) */
@@ -639,19 +648,16 @@ krb5_adm_connect(kcontext, user, prompt, opassword, sockp, ctxp,
     /*
      * Find the appropriate credentials cache and set up our identity.
      */
-    if (kret = kadm_get_ccache(kcontext, user, ccname, ccachep, &client))
+    kret = kadm_get_ccache(kcontext, user, ccname, ccachep, &client);
+    if (kret)
 	goto cleanup;
 
     /*
      * Get initial credentials.
      */
-    if (kret = kadm_get_creds(kcontext,
-			      *ccachep,
-			      client,
-			      &creds,
-			      prompt,
-			      opassword,
-			      tlife))
+    kret = kadm_get_creds(kcontext, *ccachep, client, &creds,
+			      prompt, opassword, tlife);
+    if (kret)
 	goto cleanup;
 
     /*
@@ -664,17 +670,17 @@ krb5_adm_connect(kcontext, user, prompt, opassword, sockp, ctxp,
     server_realm.length = client->realm.length;
     memcpy(server_realm.data, client->realm.data, server_realm.length);
     server_realm.data[server_realm.length] = '\0';
-    if (kret = kadm_contact_server(kcontext,
-				   &server_realm,
-				   sockp,
-				   &local_addr,
-				   &remote_addr))
+
+    kret = kadm_contact_server(kcontext, &server_realm, sockp,
+			       &local_addr, &remote_addr);
+    if (kret)
 	goto cleanup;
 
     /*
      * Obtain our authorization context
      */
-    if (kret = kadm_get_auth(kcontext, ctxp, local_addr, remote_addr))
+    kret = kadm_get_auth(kcontext, ctxp, local_addr, remote_addr);
+    if (kret)
 	goto cleanup;
 
     /*
@@ -682,21 +688,21 @@ krb5_adm_connect(kcontext, user, prompt, opassword, sockp, ctxp,
      */
     suppl_data.data = NULL;
     suppl_data.length = 0;
-    if (kret = krb5_mk_req_extended(kcontext,
-				    ctxp,
-				    AP_OPTS_MUTUAL_REQUIRED,
-				    &suppl_data,
-				    &creds,
-				    &request_data))
+
+    kret = krb5_mk_req_extended(kcontext, ctxp, AP_OPTS_MUTUAL_REQUIRED,
+				&suppl_data, &creds, &request_data);
+    if (kret)
 	goto cleanup;
 
-    if (kret = krb5_write_message(kcontext, sockp, &request_data))
+    kret = krb5_write_message(kcontext, sockp, &request_data);
+    if (kret)
 	goto cleanup;
 
     /*
      * Now read back the response.
      */
-    if (kret = krb5_read_message(kcontext, sockp, &response_data)) {
+    kret = krb5_read_message(kcontext, sockp, &response_data);
+    if (kret) {
 	goto cleanup;
     }
     else {
@@ -745,7 +751,7 @@ krb5_adm_connect(kcontext, user, prompt, opassword, sockp, ctxp,
  * If ccache is supplied, then it is destroyed.  Otherwise, the ccache is
  * the caller's responsibility to close.
  */
-void INTERFACE
+void KRB5_CALLCONV
 krb5_adm_disconnect(kcontext, socketp, auth_context, ccache)
     krb5_context	kcontext;
     int			*socketp;
