@@ -44,18 +44,26 @@
 #include <netinet/in.h>
 #endif
 
-void usage PROTOTYPE((char *));
+#ifdef KRB5_KRB4_COMPAT
+#include <des.h>
+#endif
 
-krb5_sigtype request_exit PROTOTYPE((int));
-krb5_sigtype request_hup  PROTOTYPE((int));
+#if defined(NEED_DAEMON_PROTO)
+extern int daemon(int, int);
+#endif
 
-void setup_signal_handlers PROTOTYPE((void));
+void usage (char *);
 
-krb5_error_code setup_sam PROTOTYPE((void));
+krb5_sigtype request_exit (int);
+krb5_sigtype request_hup  (int);
 
-void initialize_realms PROTOTYPE((krb5_context, int, char **));
+void setup_signal_handlers (void);
 
-void finish_realms PROTOTYPE((char *));
+krb5_error_code setup_sam (void);
+
+void initialize_realms (krb5_context, int, char **);
+
+void finish_realms (char *);
 
 static int nofork = 0;
 static int rkey_init_done = 0;
@@ -70,9 +78,7 @@ static struct sigaction s_action;
  * Find the realm entry for a given realm.
  */
 kdc_realm_t *
-find_realm_data(rname, rsize)
-    char 	*rname;
-    krb5_ui_4	rsize;
+find_realm_data(char *rname, krb5_ui_4 rsize)
 {
     int i;
     for (i=0; i<kdc_numrealms; i++) {
@@ -84,8 +90,7 @@ find_realm_data(rname, rsize)
 }
 
 krb5_error_code
-setup_server_realm(sprinc)
-    krb5_principal	sprinc;
+setup_server_realm(krb5_principal sprinc)
 {
     krb5_error_code	kret;
     kdc_realm_t		*newrealm;
@@ -104,8 +109,7 @@ setup_server_realm(sprinc)
 }
 
 static void
-finish_realm(rdp)
-    kdc_realm_t *rdp;
+finish_realm(kdc_realm_t *rdp)
 {
     if (rdp->realm_dbname)
 	free(rdp->realm_dbname);
@@ -115,8 +119,8 @@ finish_realm(rdp)
 	free(rdp->realm_stash);
     if (rdp->realm_ports)
 	free(rdp->realm_ports);
-    if (rdp->realm_kstypes)
-	free(rdp->realm_kstypes);
+    if (rdp->realm_tcp_ports)
+	free(rdp->realm_tcp_ports);
     if (rdp->realm_keytab)
 	krb5_kt_close(rdp->realm_context, rdp->realm_keytab);
     if (rdp->realm_context) {
@@ -126,16 +130,13 @@ finish_realm(rdp)
 	    memset(rdp->realm_mkey.contents, 0, rdp->realm_mkey.length);
 	    free(rdp->realm_mkey.contents);
 	}
-	if (rdp->realm_tgskey.length && rdp->realm_tgskey.contents) {
-	    memset(rdp->realm_tgskey.contents, 0, rdp->realm_tgskey.length);
-	    free(rdp->realm_tgskey.contents);
-	}
 	krb5_db_fini(rdp->realm_context);
 	if (rdp->realm_tgsprinc)
 	    krb5_free_principal(rdp->realm_context, rdp->realm_tgsprinc);
 	krb5_free_context(rdp->realm_context);
     }
     memset((char *) rdp, 0, sizeof(*rdp));
+    free(rdp);
 }
 
 /*
@@ -146,30 +147,14 @@ finish_realm(rdp)
  * realm data and we should be all set to begin operation for that realm.
  */
 static krb5_error_code
-init_realm(progname, rdp, realm, def_dbname, def_mpname,
-		 def_enctype, def_ports, def_manual)
-    char		*progname;
-    kdc_realm_t		*rdp;
-    char		*realm;
-    char		*def_dbname;
-    char		*def_mpname;
-    krb5_enctype	def_enctype;
-    char		*def_ports;
-    krb5_boolean	def_manual;
+init_realm(char *progname, kdc_realm_t *rdp, char *realm, char *def_dbname,
+	   char *def_mpname, krb5_enctype def_enctype, char *def_udp_ports,
+	   char *def_tcp_ports, krb5_boolean def_manual)
 {
     krb5_error_code	kret;
     krb5_boolean	manual;
-    krb5_db_entry	db_entry;
-    int			num2get;
-    krb5_boolean	more;
-    krb5_boolean	db_inited;
     krb5_realm_params	*rparams;
-    krb5_key_data	*kdata;
-    krb5_key_salt_tuple	*kslist;
-    krb5_int32		nkslist;
-    int			i;
 
-    db_inited = 0;
     memset((char *) rdp, 0, sizeof(kdc_realm_t));
     if (!realm) {
 	kret = EINVAL;
@@ -209,12 +194,16 @@ init_realm(progname, rdp, realm, def_dbname, def_mpname,
 	rdp->realm_mpname = (def_mpname) ? strdup(def_mpname) :
 	    strdup(KRB5_KDB_M_NAME);
 
-    /* Handle KDC port */
+    /* Handle KDC ports */
     if (rparams && rparams->realm_kdc_ports)
 	rdp->realm_ports = strdup(rparams->realm_kdc_ports);
     else
-	rdp->realm_ports = strdup(def_ports);
-	    
+	rdp->realm_ports = strdup(def_udp_ports);
+    if (rparams && rparams->realm_kdc_tcp_ports)
+	rdp->realm_tcp_ports = strdup(rparams->realm_kdc_tcp_ports);
+    else
+	rdp->realm_tcp_ports = strdup(def_tcp_ports);
+
     /* Handle stash file */
     if (rparams && rparams->realm_stash_file) {
 	rdp->realm_stash = strdup(rparams->realm_stash_file);
@@ -240,35 +229,7 @@ init_realm(progname, rdp, realm, def_dbname, def_mpname,
 
     /* Handle ticket renewable maximum life */
     rdp->realm_maxrlife = (rparams && rparams->realm_max_rlife_valid) ?
-	rparams->realm_max_rlife : KRB5_KDB_MAX_LIFE;
-
-    /* Handle key/salt list */
-    if (rparams && rparams->realm_num_keysalts) {
-	rdp->realm_kstypes = rparams->realm_keysalts;
-	rdp->realm_nkstypes = rparams->realm_num_keysalts;
-	rparams->realm_keysalts = NULL;
-	rparams->realm_num_keysalts = 0;
-	kslist = (krb5_key_salt_tuple *) rdp->realm_kstypes;
-	nkslist = rdp->realm_nkstypes;
-    } else {
-	/*
-	 * XXX  Initialize default key/salt list.
-	 */
-	if ((kslist = (krb5_key_salt_tuple *)
-	     malloc(sizeof(krb5_key_salt_tuple)))) {
-	    kslist->ks_enctype = ENCTYPE_DES_CBC_CRC;
-	    kslist->ks_salttype = KRB5_KDB_SALTTYPE_NORMAL;
-	    rdp->realm_kstypes = kslist;
-	    rdp->realm_nkstypes = 1;
-	    nkslist = 1;
-	}
-	else {
-	    com_err(progname, ENOMEM,
-		    "while setting up key/salt list for realm %s",
-		    realm);
-	    exit(1);
-	}
-    }
+	rparams->realm_max_rlife : KRB5_KDB_MAX_RLIFE;
 
     if (rparams)
 	krb5_free_realm_params(rdp->realm_context, rparams);
@@ -319,8 +280,7 @@ init_realm(progname, rdp, realm, def_dbname, def_mpname,
 	com_err(progname, kret,
 		"while initializing database for realm %s", realm);
 	goto whoops;
-    } else
-	db_inited = 1;
+    }
 
     /* Verify the master key */
     if ((kret = krb5_db_verify_master_key(rdp->realm_context,
@@ -331,51 +291,6 @@ init_realm(progname, rdp, realm, def_dbname, def_mpname,
 	goto whoops;
     }
 
-    /* Fetch the master key and get its version number */
-    num2get = 1;
-    kret = krb5_db_get_principal(rdp->realm_context, rdp->realm_mprinc,
-				 &db_entry, &num2get, &more);
-    if (!kret) {
-	if (num2get != 1)
-	    kret = KRB5_KDB_NOMASTERKEY;
-	else {
-	    if (more) {
-		krb5_db_free_principal(rdp->realm_context,
-				       &db_entry,
-				       num2get);
-		kret = KRB5KDC_ERR_PRINCIPAL_NOT_UNIQUE;
-	    }
-	}
-    }
-    if (kret) {
-	com_err(progname, kret,
-		"while fetching master entry for realm %s", realm);
-	goto whoops;
-    }
-	
-    /*
-     * Get the most recent master key.  Search the key list in
-     * the order specified by the key/salt list.
-     */
-    kdata = (krb5_key_data *) NULL;
-    for (i=0; i<nkslist; i++) {
-	if (!(kret = krb5_dbe_find_enctype(rdp->realm_context,
-					   &db_entry,
-					   kslist[i].ks_enctype,
-					   -1,
-					   -1,
-					   &kdata)))
-	    break;
-    }
-    if (!kdata) {
-	com_err(progname, kret,
-		"while finding master key for realm %s",
-		realm);
-	goto whoops;
-    }
-    rdp->realm_mkvno = kdata->key_data_kvno;
-    krb5_db_free_principal(rdp->realm_context, &db_entry, num2get);
-
     if ((kret = krb5_db_set_mkey(rdp->realm_context, &rdp->realm_mkey))) {
 	com_err(progname, kret,
 		"while setting master key for realm %s", realm);
@@ -383,7 +298,7 @@ init_realm(progname, rdp, realm, def_dbname, def_mpname,
     }
 
     /* Set up the keytab */
-    if ((kret = krb5_ktkdb_resolve(rdp->realm_context, 
+    if ((kret = krb5_ktkdb_resolve(rdp->realm_context, NULL,
 				   &rdp->realm_keytab))) {
 	com_err(progname, kret,
 		"while resolving kdb keytab for realm %s", realm);
@@ -399,65 +314,7 @@ init_realm(progname, rdp, realm, def_dbname, def_mpname,
 	goto whoops;
     }
 
-    /* Get the TGS database entry */
-    num2get = 1;
-    if (!(kret = krb5_db_get_principal(rdp->realm_context,
-				       rdp->realm_tgsprinc,
-				       &db_entry,
-				       &num2get,
-				       &more))) {
-	if (num2get != 1)
-	    kret = KRB5KDC_ERR_S_PRINCIPAL_UNKNOWN;
-	else {
-	    if (more) {
-		krb5_db_free_principal(rdp->realm_context,
-				       &db_entry,
-				       num2get);
-		kret = KRB5KDC_ERR_PRINCIPAL_NOT_UNIQUE;
-	    }
-	}
-    }
-    if (kret) {
-	com_err(progname, kret,
-		"while fetching TGS entry for realm %s", realm);
-	goto whoops;
-    } 
-    /*
-     * Get the most recent TGS key.  Search the key list in
-     * the order specified by the key/salt list.
-     */
-    kdata = (krb5_key_data *) NULL;
-    for (i=0; i<nkslist; i++) {
-	if (!(kret = krb5_dbe_find_enctype(rdp->realm_context,
-					   &db_entry,
-					   kslist[i].ks_enctype,
-					   -1,
-					   -1,
-					   &kdata)))
-	    break;
-    }
-    if (!kdata) {
-	com_err(progname, kret, "while finding TGS key for realm %s",
-		realm);
-	goto whoops;
-    }
-    if (!(kret = krb5_dbekd_decrypt_key_data(rdp->realm_context,
-					     &rdp->realm_mkey,
-					     kdata,
-					     &rdp->realm_tgskey, NULL))){
-	rdp->realm_tgskvno = kdata->key_data_kvno;
-    }
-    krb5_db_free_principal(rdp->realm_context,
-			   &db_entry,
-			   num2get);
-    if (kret) {
-	com_err(progname, kret,
-		"while decrypting TGS key for realm %s", realm);
-	goto whoops;
-    }
-
     if (!rkey_init_done) {
-	krb5_timestamp now;
 	krb5_data seed;
 #ifdef KRB5_KRB4_COMPAT
 	krb5_keyblock temp_key;
@@ -467,17 +324,11 @@ init_realm(progname, rdp, realm, def_dbname, def_mpname,
 	 * generators.
 	 */
 
-	if ((kret = krb5_timeofday(rdp->realm_context, &now)))
-	    goto whoops;
-	seed.length = sizeof(now);
-	seed.data = (char *) &now;
-	if ((kret = krb5_c_random_seed(rdp->realm_context, &seed)))
-	    goto whoops;
-
 	seed.length = rdp->realm_mkey.length;
 	seed.data = rdp->realm_mkey.contents;
 
-	if ((kret = krb5_c_random_seed(rdp->realm_context, &seed)))
+	if ((kret = krb5_c_random_add_entropy(rdp->realm_context,
+					     KRB5_C_RANDSOURCE_TRUSTEDPARTY, &seed)))
 	    goto whoops;
 
 #ifdef KRB5_KRB4_COMPAT
@@ -498,14 +349,14 @@ init_realm(progname, rdp, realm, def_dbname, def_mpname,
      * If we choked, then clean up any dirt we may have dropped on the floor.
      */
     if (kret) {
+        
 	finish_realm(rdp);
     }
     return(kret);
 }
 
 krb5_sigtype
-request_exit(signo)
-    int signo;
+request_exit(int signo)
 {
     signal_requests_exit = 1;
 
@@ -517,8 +368,7 @@ request_exit(signo)
 }
 
 krb5_sigtype
-request_hup(signo)
-    int signo;
+request_hup(int signo)
 {
     signal_requests_hup = 1;
 
@@ -530,7 +380,7 @@ request_hup(signo)
 }
 
 void
-setup_signal_handlers()
+setup_signal_handlers(void)
 {
 #ifdef POSIX_SIGNALS
     (void) sigemptyset(&s_action.sa_mask);
@@ -550,24 +400,20 @@ setup_signal_handlers()
 }
 
 krb5_error_code
-setup_sam()
+setup_sam(void)
 {
     return krb5_c_make_random_key(kdc_context, ENCTYPE_DES_CBC_MD5, &psr_key);
 }
 
 void
-usage(name)
-char *name;
+usage(char *name)
 {
     fprintf(stderr, "usage: %s [-d dbpathname] [-r dbrealmname] [-R replaycachename ]\n\t[-m] [-k masterenctype] [-M masterkeyname] [-p port] [-4 v4mode] [-X] [-n]\n", name);
     return;
 }
 
 void
-initialize_realms(kcontext, argc, argv)
-    krb5_context 	kcontext;
-    int			argc;
-    char		**argv;
+initialize_realms(krb5_context kcontext, int argc, char **argv)
 {
     int 		c;
     char		*db_name = (char *) NULL;
@@ -578,7 +424,8 @@ initialize_realms(kcontext, argc, argv)
     krb5_enctype	menctype = ENCTYPE_UNKNOWN;
     kdc_realm_t		*rdatap;
     krb5_boolean	manual = FALSE;
-    char		*default_ports = 0;
+    char		*default_udp_ports = 0;
+    char		*default_tcp_ports = 0;
     krb5_pointer	aprof;
     const char		*hierarchy[3];
 #ifdef KRB5_KRB4_COMPAT
@@ -594,8 +441,11 @@ initialize_realms(kcontext, argc, argv)
 	hierarchy[0] = "kdcdefaults";
 	hierarchy[1] = "kdc_ports";
 	hierarchy[2] = (char *) NULL;
-	if (krb5_aprof_get_string(aprof, hierarchy, TRUE, &default_ports))
-	    default_ports = 0;
+	if (krb5_aprof_get_string(aprof, hierarchy, TRUE, &default_udp_ports))
+	    default_udp_ports = 0;
+	hierarchy[1] = "kdc_tcp_ports";
+	if (krb5_aprof_get_string(aprof, hierarchy, TRUE, &default_tcp_ports))
+	    default_tcp_ports = 0;
 #ifdef KRB5_KRB4_COMPAT
 	hierarchy[1] = "v4_mode";
 	if (krb5_aprof_get_string(aprof, hierarchy, TRUE, &v4mode))
@@ -605,8 +455,10 @@ initialize_realms(kcontext, argc, argv)
 	if (aprof)
 	     krb5_aprof_finish(aprof);
     }
-    if (default_ports == 0)
-	default_ports = strdup(DEFAULT_KDC_PORTLIST);
+    if (default_udp_ports == 0)
+	default_udp_ports = strdup(DEFAULT_KDC_UDP_PORTLIST);
+    if (default_tcp_ports == 0)
+	default_tcp_ports = strdup(DEFAULT_KDC_TCP_PORTLIST);
     /*
      * Loop through the option list.  Each time we encounter a realm name,
      * use the previously scanned options to fill in for defaults.
@@ -618,8 +470,9 @@ initialize_realms(kcontext, argc, argv)
 		if ((rdatap = (kdc_realm_t *) malloc(sizeof(kdc_realm_t)))) {
 		    if ((retval = init_realm(argv[0], rdatap, optarg, db_name,
 					     mkey_name, menctype,
-					     default_ports, manual))) {
-			fprintf(stderr,"%s: cannot initialize realm %s\n",
+					     default_udp_ports,
+					     default_tcp_ports, manual))) {
+			fprintf(stderr,"%s: cannot initialize realm %s - see log file for details\n",
 				argv[0], optarg);
 			exit(1);
 		    }
@@ -650,9 +503,14 @@ initialize_realms(kcontext, argc, argv)
 	    rcname = optarg;
 	    break;
 	case 'p':
-	    if (default_ports)
-		free(default_ports);
-	    default_ports = strdup(optarg);
+	    if (default_udp_ports)
+		free(default_udp_ports);
+	    default_udp_ports = strdup(optarg);
+#if 0 /* not yet */
+	    if (default_tcp_ports)
+		free(default_tcp_ports);
+	    default_tcp_ports = strdup(optarg);
+#endif
 	    break;
 	case '4':
 #ifdef KRB5_KRB4_COMPAT
@@ -701,13 +559,15 @@ initialize_realms(kcontext, argc, argv)
 	if ((retval = krb5_get_default_realm(kcontext, &lrealm))) {
 	    com_err(argv[0], retval,
 		    "while attempting to retrieve default realm");
+	    fprintf (stderr, "%s: %s, attempting to retrieve default realm\n",
+		     argv[0], error_message (retval));
 	    exit(1);
 	}
 	if ((rdatap = (kdc_realm_t *) malloc(sizeof(kdc_realm_t)))) {
 	    if ((retval = init_realm(argv[0], rdatap, lrealm, db_name,
-				     mkey_name, menctype, default_ports,
-				     manual))) {
-		fprintf(stderr,"%s: cannot initialize realm %s\n",
+				     mkey_name, menctype, default_udp_ports,
+				     default_tcp_ports, manual))) {
+		fprintf(stderr,"%s: cannot initialize realm %s - see log file for details\n",
 			argv[0], lrealm);
 		exit(1);
 	    }
@@ -721,27 +581,31 @@ initialize_realms(kcontext, argc, argv)
      * Now handle the replay cache.
      */
     if ((retval = kdc_initialize_rcache(kcontext, rcname))) {
-	com_err(argv[0], retval, "while initializing KDC replay cache");
+	com_err(argv[0], retval, "while initializing KDC replay cache '%s'",
+		rcname);
 	exit(1);
     }
 #endif
 
     /* Ensure that this is set for our first request. */
     kdc_active_realm = kdc_realmlist[0];
-    if (default_ports)
-	free(default_ports);
+    if (default_udp_ports)
+	free(default_udp_ports);
+    if (default_tcp_ports)
+	free(default_tcp_ports);
 
     return;
 }
 
 void
-finish_realms(prog)
-    char *prog;
+finish_realms(char *prog)
 {
     int i;
 
-    for (i = 0; i < kdc_numrealms; i++)
+    for (i = 0; i < kdc_numrealms; i++) {
 	finish_realm(kdc_realmlist[i]);
+	kdc_realmlist[i] = 0;
+    }
 }
 
 /*
@@ -770,13 +634,10 @@ finish_realms(prog)
  exit
  */
 
-int main(argc, argv)
-     int argc;
-     char *argv[];
+int main(int argc, char **argv)
 {
     krb5_error_code	retval;
     krb5_context	kcontext;
-    int			*port_list;
     int errout = 0;
 
     if (strrchr(argv[0], '/'))
@@ -789,7 +650,6 @@ int main(argc, argv)
     }
     memset((char *) kdc_realmlist, 0,
 	   (size_t) (sizeof(kdc_realm_t *) * KRB5_KDC_MAX_REALMS));
-    port_list = NULL;
 
     /*
      * A note about Kerberos contexts: This context, "kcontext", is used
@@ -803,6 +663,9 @@ int main(argc, argv)
 	    exit(1);
     }
     krb5_klog_init(kcontext, "kdc", argv[0], 1);
+    /* N.B.: After this point, com_err sends output to the KDC log
+       file, and not to stderr.  */
+
     initialize_kdc5_error_table();
 
     /*
@@ -812,7 +675,8 @@ int main(argc, argv)
 
     setup_signal_handlers();
 
-    if (retval = setup_sam()) {
+    retval = setup_sam();
+    if (retval) {
 	com_err(argv[0], retval, "while initializing SAM");
 	finish_realms(argv[0]);
 	return 1;
@@ -840,6 +704,14 @@ int main(argc, argv)
     krb5_klog_syslog(LOG_INFO, "shutting down");
     krb5_klog_close(kdc_context);
     finish_realms(argv[0]);
+    if (kdc_realmlist) 
+      free(kdc_realmlist);
+#ifdef USE_RCACHE
+    (void) krb5_rc_close(kcontext, kdc_rcache);
+#endif
+#ifndef NOCACHE
+    kdc_free_lookaside(kcontext);
+#endif
     krb5_free_context(kcontext);
     return errout;
 }
