@@ -24,10 +24,10 @@
 #define _GSSAPIP_GENERIC_H_
 
 /*
- * $Id: gssapiP_generic.h,v 1.24.10.2 2001/10/30 01:18:32 meeroh Exp $
+ * $Id: gssapiP_generic.h,v 1.34.2.1 2003/12/16 02:56:16 tlyu Exp $
  */
 
-#if (defined(_MSDOS) || defined(_WIN32) || defined(macintosh))
+#if defined(_WIN32)
 #include "k5-int.h"
 #else
 #ifdef HAVE_STDLIB_H
@@ -40,11 +40,14 @@
 #include "gssapi_err_generic.h"
 #include <errno.h>
 
+#include "k5-platform.h"
+typedef UINT64_TYPE gssint_uint64;
+
 /** helper macros **/
 
 #define g_OID_equal(o1,o2) \
    (((o1)->length == (o2)->length) && \
-    (memcmp((o1)->elements,(o2)->elements,(int) (o1)->length) == 0))
+    (memcmp((o1)->elements,(o2)->elements,(unsigned int) (o1)->length) == 0))
 
 /* this code knows that an int on the wire is 32 bits.  The type of
    num should be at least this big, or the extra shifts may do weird
@@ -131,51 +134,53 @@
 
 typedef struct _g_set *g_set;
 
-int g_set_init PROTOTYPE((g_set *s));
-int g_set_destroy PROTOTYPE((g_set *s));
-int g_set_entry_add PROTOTYPE((g_set *s, void *key, void *value));
-int g_set_entry_delete PROTOTYPE((g_set *s, void *key));
-int g_set_entry_get PROTOTYPE((g_set *s, void *key, void **value));
+int g_set_init (g_set *s);
+int g_set_destroy (g_set *s);
+int g_set_entry_add (g_set *s, void *key, void *value);
+int g_set_entry_delete (g_set *s, void *key);
+int g_set_entry_get (g_set *s, void *key, void **value);
 
-int g_save_name PROTOTYPE((void **vdb, gss_name_t *name));
-int g_save_cred_id PROTOTYPE((void **vdb, gss_cred_id_t *cred));
-int g_save_ctx_id PROTOTYPE((void **vdb, gss_ctx_id_t *ctx));
+int g_save_name (void **vdb, gss_name_t *name);
+int g_save_cred_id (void **vdb, gss_cred_id_t *cred);
+int g_save_ctx_id (void **vdb, gss_ctx_id_t *ctx);
 
-int g_validate_name PROTOTYPE((void **vdb, gss_name_t *name));
-int g_validate_cred_id PROTOTYPE((void **vdb, gss_cred_id_t *cred));
-int g_validate_ctx_id PROTOTYPE((void **vdb, gss_ctx_id_t *ctx));
+int g_validate_name (void **vdb, gss_name_t *name);
+int g_validate_cred_id (void **vdb, gss_cred_id_t *cred);
+int g_validate_ctx_id (void **vdb, gss_ctx_id_t *ctx);
 
-int g_delete_name PROTOTYPE((void **vdb, gss_name_t *name));
-int g_delete_cred_id PROTOTYPE((void **vdb, gss_cred_id_t *cred));
-int g_delete_ctx_id PROTOTYPE((void **vdb, gss_ctx_id_t *ctx));
+int g_delete_name (void **vdb, gss_name_t *name);
+int g_delete_cred_id (void **vdb, gss_cred_id_t *cred);
+int g_delete_ctx_id (void **vdb, gss_ctx_id_t *ctx);
 
-int g_make_string_buffer PROTOTYPE((const char *str, gss_buffer_t buffer));
+int g_make_string_buffer (const char *str, gss_buffer_t buffer);
 
-int g_copy_OID_set PROTOTYPE((const gss_OID_set_desc * const in, gss_OID_set *out));
+int g_copy_OID_set (const gss_OID_set_desc * const in, gss_OID_set *out);
 
-int g_token_size PROTOTYPE((gss_OID mech, unsigned int body_size));
+unsigned int g_token_size (gss_OID mech, unsigned int body_size);
 
-void g_make_token_header PROTOTYPE((gss_OID mech, int body_size,
-			  unsigned char **buf, int tok_type));
+void g_make_token_header (gss_OID mech, unsigned int body_size,
+			  unsigned char **buf, int tok_type);
 
-gss_int32 g_verify_token_header PROTOTYPE((gss_OID mech, int *body_size,
-			  unsigned char **buf, int tok_type, int toksize));
+gss_int32 g_verify_token_header (gss_OID mech, unsigned int *body_size,
+				 unsigned char **buf, int tok_type, 
+				 unsigned int toksize_in,
+				 int wrapper_required);
 
-OM_uint32 g_display_major_status PROTOTYPE((OM_uint32 *minor_status,
+OM_uint32 g_display_major_status (OM_uint32 *minor_status,
 				 OM_uint32 status_value,
 				 OM_uint32 *message_context,
-				 gss_buffer_t status_string));
+				 gss_buffer_t status_string);
 
-OM_uint32 g_display_com_err_status PROTOTYPE((OM_uint32 *minor_status,
+OM_uint32 g_display_com_err_status (OM_uint32 *minor_status,
 				   OM_uint32 status_value,
-				   gss_buffer_t status_string));
+				   gss_buffer_t status_string);
 
-gss_int32 g_order_init PROTOTYPE((void **queue, OM_uint32 seqnum,
-				  int do_replay, int do_sequence));
+gss_int32 g_order_init (void **queue, gssint_uint64 seqnum,
+				  int do_replay, int do_sequence, int wide);
 
-gss_int32 g_order_check PROTOTYPE((void **queue, OM_uint32 seqnum));
+gss_int32 g_order_check (void **queue, gssint_uint64 seqnum);
 
-void g_order_free PROTOTYPE((void **queue));
+void g_order_free (void **queue);
 
 gss_uint32 g_queue_size(void *vqueue, size_t *sizep);
 gss_uint32 g_queue_externalize(void *vqueue, unsigned char **buf,
@@ -183,62 +188,59 @@ gss_uint32 g_queue_externalize(void *vqueue, unsigned char **buf,
 gss_uint32 g_queue_internalize(void **vqueue, unsigned char **buf,
 			       size_t *lenremain);
 
-char *g_canonicalize_host PROTOTYPE((char *hostname));
-char *g_local_host_name PROTOTYPE((void));
-
-char *g_strdup PROTOTYPE((char *str));
+char *g_strdup (char *str);
 
 /** declarations of internal name mechanism functions **/
 
 OM_uint32 generic_gss_release_buffer
-PROTOTYPE((OM_uint32*,       /* minor_status */
+(OM_uint32*,       /* minor_status */
             gss_buffer_t      /* buffer */
-           ));
+           );
 
 OM_uint32 generic_gss_release_oid_set
-PROTOTYPE((OM_uint32*,       /* minor_status */
+(OM_uint32*,       /* minor_status */
             gss_OID_set*      /* set */
-           ));
+           );
 
 OM_uint32 generic_gss_release_oid
-PROTOTYPE((OM_uint32*,       /* minor_status */
+(OM_uint32*,       /* minor_status */
             gss_OID*         /* set */
-           ));
+           );
 
 OM_uint32 generic_gss_copy_oid
-PROTOTYPE( (OM_uint32 *,	/* minor_status */
+(OM_uint32 *,	/* minor_status */
 	    gss_OID,		/* oid */
 	    gss_OID *		/* new_oid */
-	    ));
+	    );
 
 OM_uint32 generic_gss_create_empty_oid_set
-PROTOTYPE( (OM_uint32 *,	/* minor_status */
+(OM_uint32 *,	/* minor_status */
 	    gss_OID_set *	/* oid_set */
-	   ));
+	   );
 
 OM_uint32 generic_gss_add_oid_set_member
-PROTOTYPE( (OM_uint32 *,	/* minor_status */
+(OM_uint32 *,	/* minor_status */
 	    gss_OID,		/* member_oid */
 	    gss_OID_set *	/* oid_set */
-	   ));
+	   );
 
 OM_uint32 generic_gss_test_oid_set_member
-PROTOTYPE( (OM_uint32 *,	/* minor_status */
+(OM_uint32 *,	/* minor_status */
 	    gss_OID,		/* member */
 	    gss_OID_set,	/* set */
 	    int *		/* present */
-	   ));
+	   );
 
 OM_uint32 generic_gss_oid_to_str
-PROTOTYPE( (OM_uint32 *,	/* minor_status */
+(OM_uint32 *,	/* minor_status */
 	    gss_OID,		/* oid */
 	    gss_buffer_t	/* oid_str */
-	   ));
+	   );
 
 OM_uint32 generic_gss_str_to_oid
-PROTOTYPE( (OM_uint32 *,	/* minor_status */
+(OM_uint32 *,	/* minor_status */
 	    gss_buffer_t,	/* oid_str */
 	    gss_OID *		/* oid */
-	   ));
+	   );
 
 #endif /* _GSSAPIP_GENERIC_H_ */
