@@ -619,4 +619,12 @@ krb5_error_code gss_krb5int_make_seal_token_v3(krb5_context,
 					       gss_buffer_t,
 					       int, int);
 
+OM_uint32 gss_krb5int_unseal_token_v3(krb5_context *contextptr,
+				      OM_uint32 *minor_status,
+				      krb5_gss_ctx_id_rec *ctx,
+				      unsigned char *ptr, int bodysize,
+				      gss_buffer_t message_buffer,
+				      int *conf_state, int *qop_state, 
+				      int toktype);
+
 #endif /* _GSSAPIP_KRB5_H_ */
