@@ -21,7 +21,7 @@
  */
 
 /*
- * $Id: indicate_mechs.c,v 1.11 1999/03/26 03:51:43 tytso Exp $
+ * $Id: indicate_mechs.c,v 1.12 2002/07/01 20:48:34 raeburn Exp $
  */
 
 #include "gssapiP_krb5.h"
@@ -33,7 +33,7 @@ krb5_gss_indicate_mechs(minor_status, mech_set)
 {
    *minor_status = 0;
 
-   if (! g_copy_OID_set(gss_mech_set_krb5_v1v2, mech_set)) {
+   if (! g_copy_OID_set(gss_mech_set_krb5_both, mech_set)) {
          *mech_set     = GSS_C_NO_OID_SET;
          *minor_status = ENOMEM;
          return(GSS_S_FAILURE);
