@@ -70,13 +70,13 @@
 #include "kdb_db2.h"
 
 static char *gen_dbsuffix 
-	PROTOTYPE((char *, char * ));
+	(char *, char * );
 static krb5_error_code krb5_db2_db_start_update 
-	PROTOTYPE((krb5_context));
+	(krb5_context);
 static krb5_error_code krb5_db2_db_end_update 
-	PROTOTYPE((krb5_context));
+	(krb5_context);
 static krb5_error_code krb5_db2_db_set_hashfirst
-	PROTOTYPE((krb5_context, int));
+	(krb5_context, int);
 
 static char default_db_name[] = DEFAULT_KDB_FILE;
 
@@ -649,14 +649,16 @@ krb5_db2_db_create(context, db_name, flags)
 /*
  * Destroy the database.  Zero's out all of the files, just to be sure.
  */
-krb5_error_code
+static krb5_error_code
 destroy_file_suffix(dbname, suffix)
     char *dbname;
     char *suffix;
 {
     char *filename;
     struct stat statb;
-    int nb,fd,i,j;
+    int nb,fd;
+    unsigned int j;
+    off_t pos;
     char buf[BUFSIZ];
     char zbuf[BUFSIZ];
     int dowrite;
@@ -685,8 +687,8 @@ destroy_file_suffix(dbname, suffix)
      * we're just about to unlink it anyways.
      */
     memset(zbuf, 0, BUFSIZ);
-    i = 0;
-    while (i < statb.st_size) {
+    pos = 0;
+    while (pos < statb.st_size) {
 	dowrite = 0;
 	nb = read(fd, buf, BUFSIZ);
 	if (nb < 0) {
@@ -700,16 +702,18 @@ destroy_file_suffix(dbname, suffix)
 		break;
 	    }
 	}
+	/* For signedness */
+	j = nb;
 	if (dowrite) {
-	    lseek(fd, i, SEEK_SET);
-	    nb = write(fd, zbuf, nb);
+	    lseek(fd, pos, SEEK_SET);
+	    nb = write(fd, zbuf, j);
 	    if (nb < 0) {
 		int retval = errno;
 		free(filename);
 		return retval;
 	    }
 	}
-	i += nb;
+	pos += nb;
     }
     /* ??? Is fsync really needed?  I don't know of any non-networked
        filesystem which will discard queued writes to disk if a file
@@ -1087,7 +1091,7 @@ krb5_db2_db_delete_principal(context, searchfor, nentries)
     for (i = 0; i < entry.n_key_data; i++) {
 	if (entry.key_data[i].key_data_length[0]) {
 	    memset((char *)entry.key_data[i].key_data_contents[0], 0, 
-		   entry.key_data[i].key_data_length[0]); 
+		   (unsigned) entry.key_data[i].key_data_length[0]); 
 	}
     }
 
@@ -1115,10 +1119,11 @@ cleanup:
 }
 
 krb5_error_code
-krb5_db2_db_iterate (context, func, func_arg)
+krb5_db2_db_iterate_ext(context, func, func_arg, backwards, recursive)
     krb5_context context;
-    krb5_error_code (*func) PROTOTYPE((krb5_pointer, krb5_db_entry *));
+    krb5_error_code (*func) (krb5_pointer, krb5_db_entry *);
     krb5_pointer func_arg;
+    int backwards, recursive;
 {
     krb5_db2_context *db_ctx;
     DB *db;
@@ -1127,17 +1132,36 @@ krb5_db2_db_iterate (context, func, func_arg)
     krb5_db_entry entries;
     krb5_error_code retval;
     int dbret;
-    
+    void *cookie;
+
+    cookie = NULL;
     if (!k5db2_inited(context))
 	return KRB5_KDB_DBNOTINITED;
 
     db_ctx = (krb5_db2_context *) context->db_context;
     retval = krb5_db2_db_lock(context, KRB5_LOCKMODE_SHARED);
+
     if (retval)
 	return retval;
 
     db = db_ctx->db;
-    dbret = (*db->seq)(db, &key, &contents, R_FIRST);
+    if (recursive && db->type != DB_BTREE) {
+	(void)krb5_db2_db_unlock(context);
+	return KRB5_KDB_UK_RERROR; /* Not optimal, but close enough. */
+    }
+
+    if (!recursive) {
+	dbret = (*db->seq)(db, &key, &contents,
+			   backwards ? R_LAST : R_FIRST);
+    } else {
+#ifdef HAVE_BT_RSEQ
+	dbret = bt_rseq(db, &key, &contents, &cookie,
+			backwards ? R_LAST : R_FIRST);
+#else
+	(void)krb5_db2_db_unlock(context);
+	return KRB5_KDB_UK_RERROR; /* Not optimal, but close enough. */
+#endif
+    }
     while (dbret == 0) {
 	contdata.data = contents.data;
 	contdata.length = contents.size;
@@ -1148,7 +1172,18 @@ krb5_db2_db_iterate (context, func, func_arg)
 	krb5_dbe_free_contents(context, &entries);
 	if (retval)
 	    break;
-	dbret = (*db->seq)(db, &key, &contents, R_NEXT);
+	if (!recursive) {
+	    dbret = (*db->seq)(db, &key, &contents,
+			       backwards ? R_PREV : R_NEXT);
+	} else {
+#ifdef HAVE_BT_RSEQ
+	    dbret = bt_rseq(db, &key, &contents, &cookie,
+			    backwards ? R_PREV : R_NEXT);
+#else
+	    (void)krb5_db2_db_unlock(context);
+	    return KRB5_KDB_UK_RERROR; /* Not optimal, but close enough. */
+#endif
+	}
     }
     switch (dbret) {
     case 1:
@@ -1162,6 +1197,15 @@ krb5_db2_db_iterate (context, func, func_arg)
     return retval;
 }
 
+krb5_error_code
+krb5_db2_db_iterate(context, func, func_arg)
+    krb5_context context;
+    krb5_error_code (*func) (krb5_pointer, krb5_db_entry *);
+    krb5_pointer func_arg;
+{
+    return krb5_db2_db_iterate_ext(context, func, func_arg, 0, 0);
+}
+
 krb5_boolean
 krb5_db2_db_set_lockmode(context, mode)
     krb5_context context;
@@ -1305,6 +1349,7 @@ kdb5_context_internalize(kcontext, argp, buffer, lenremain)
     krb5_int32		lockcount;
     krb5_int32		lockmode;
     krb5_int32		dbnamelen;
+    krb5_boolean        nb_lock;
     char		*dbname;
 
     bp = *buffer;
@@ -1342,7 +1387,8 @@ kdb5_context_internalize(kcontext, argp, buffer, lenremain)
 			kret = krb5_db_lock(tmpctx, lockmode);
 		    if (!kret && lockmode)
 			dbctx->db_locks_held = lockcount;
-		    (void) krb5_db2_db_set_lockmode(tmpctx, nb_lockmode);
+		    nb_lock = nb_lockmode & 0xff;
+		    (void) krb5_db2_db_set_lockmode(tmpctx, nb_lock);
 		}
 		if (dbname)
 		    krb5_xfree(dbname);
