@@ -24,21 +24,23 @@
  * or implied warranty.
  * 
  */
+#include <string.h>
 
 #include "k5-int.h"
 #include "kdb_kt.h"
 
 static int
 is_xrealm_tgt(krb5_context, krb5_const_principal);
-krb5_error_code krb5_ktkdb_close KRB5_PROTOTYPE((krb5_context, krb5_keytab));
 
-krb5_error_code krb5_ktkdb_get_entry KRB5_PROTOTYPE((krb5_context, krb5_keytab, krb5_const_principal,
-		   krb5_kvno, krb5_enctype, krb5_keytab_entry *));
+krb5_error_code krb5_ktkdb_close (krb5_context, krb5_keytab);
+
+krb5_error_code krb5_ktkdb_get_entry (krb5_context, krb5_keytab, krb5_const_principal,
+		   krb5_kvno, krb5_enctype, krb5_keytab_entry *);
 
 krb5_kt_ops krb5_kt_kdb_ops = {
     0,
     "KDB", 	/* Prefix -- this string should not appear anywhere else! */
-    NULL,			/* resolve */
+    krb5_ktkdb_resolve,		/* resolve */
     NULL,			/* get_name */
     krb5_ktkdb_close,		/* close */
     krb5_ktkdb_get_entry,	/* get */
@@ -55,8 +57,9 @@ typedef struct krb5_ktkdb_data {
 } krb5_ktkdb_data;
 
 krb5_error_code
-krb5_ktkdb_resolve(context, id)
+krb5_ktkdb_resolve(context, name, id)
     krb5_context  	  context;
+    const char		* name;
     krb5_keytab		* id;
 {
     if ((*id = (krb5_keytab) malloc(sizeof(**id))) == NULL)
@@ -85,15 +88,32 @@ krb5_ktkdb_close(context, kt)
   return 0;
 }
 
+static krb5_context ktkdb_ctx = NULL;
+
+/*
+ * Set a different context for use with ktkdb_get_entry().  This is
+ * primarily useful for kadmind, where the gssapi library context,
+ * which will be used for the keytab, will necessarily have a
+ * different context than that used by the kadm5 library to access the
+ * database for its own purposes.
+ */
 krb5_error_code
-krb5_ktkdb_get_entry(context, id, principal, kvno, enctype, entry)
-    krb5_context 	  context;
+krb5_ktkdb_set_context(krb5_context ctx)
+{
+    ktkdb_ctx = ctx;
+    return 0;
+}
+
+krb5_error_code
+krb5_ktkdb_get_entry(in_context, id, principal, kvno, enctype, entry)
+    krb5_context 	  in_context;
     krb5_keytab 	  id;
     krb5_const_principal  principal;
     krb5_kvno 	 	  kvno;
     krb5_enctype 	  enctype;
     krb5_keytab_entry 	* entry;
 {
+    krb5_context	  context;
     krb5_keyblock       * master_key;
     krb5_error_code 	  kerror = 0;
     krb5_key_data 	* key_data;
@@ -103,6 +123,11 @@ krb5_ktkdb_get_entry(context, id, principal, kvno, enctype, entry)
     int xrealm_tgt = is_xrealm_tgt(context, principal);
     int similar;
 
+    if (ktkdb_ctx)
+	context = ktkdb_ctx;
+    else
+	context = in_context;
+
     /* Open database */
     /* krb5_db_init(context); */
     if ((kerror = krb5_db_open_database(context)))
@@ -147,19 +172,20 @@ krb5_ktkdb_get_entry(context, id, principal, kvno, enctype, entry)
     if (kerror)
 	goto error;
 
-    kerror = krb5_c_enctype_compare(context, enctype, entry->key.enctype, &similar);
-    if (kerror)
-	goto error;
+    if (enctype > 0) {	
+	kerror = krb5_c_enctype_compare(context, enctype,
+					entry->key.enctype, &similar);
+	if (kerror)
+	    goto error;
 
-    if (!similar) {
-		kerror = KRB5_KDB_NO_PERMITTED_KEY;
-	goto error;
+	if (!similar) {
+	    kerror = KRB5_KDB_NO_PERMITTED_KEY;
+	    goto error;
+	}
     }
-
     /*
      * Coerce the enctype of the output keyblock in case we got an
-     * inexact match on the enctype; this behavior will go away when
-     * the key storage architecture gets redesigned for 1.3.
+     * inexact match on the enctype.
      */
     entry->key.enctype = enctype;
 
@@ -191,7 +217,7 @@ is_xrealm_tgt(krb5_context context, krb5_const_principal princ)
     dat = krb5_princ_component(context, princ, 1);
     if (dat->length != princ->realm.length)
 	return 1;
-    if (strcmp(dat->data, princ->realm.data) == 0)
+    if (strncmp(dat->data, princ->realm.data, dat->length) == 0)
 	return 0;
     return 1;
 
