@@ -45,61 +45,81 @@
 #define		INST_SZ		40
 
 struct krb_convert {
-	char	*v4_str;
-	char	*v5_str;
-	int	flags;
+    char		*v4_str;
+    char		*v5_str;
+    unsigned int	flags : 8;
+    unsigned int	len : 8;
 };
 
 #define DO_REALM_CONVERSION 0x00000001
 
 /*
  * Kadmin doesn't do realm conversion because it's currently
- * kadmin/REALM.NAME.  It should be kadmin/kerberos.master.host, but
- * we'll fix that in the next release.
+ * kadmin/REALM.NAME.  Zephyr doesn't because it's just zephyr/zephyr.
+ *
+ * "Realm conversion" is a bit of a misnomer; really, the v5 name is
+ * using a FQDN or something that looks like it, where the v4 name is
+ * just using the first label.  Sometimes that second principal name
+ * component is a hostname, sometimes the realm name, sometimes it's
+ * neither.
+ *
+ * This list should probably be more configurable, and more than
+ * likely on a per-realm basis, so locally-defined services can be
+ * added, or not.
  */
 static const struct krb_convert sconv_list[] = {
-    {"kadmin",	"kadmin",	0},
-    {"rcmd",	"host",		DO_REALM_CONVERSION},
-    {"discuss",	"discuss",	DO_REALM_CONVERSION},
-    {"rvdsrv",	"rvdsrv",	DO_REALM_CONVERSION},
-    {"sample",	"sample",	DO_REALM_CONVERSION},
-    {"olc",	"olc",		DO_REALM_CONVERSION},
-    {"pop",	"pop",		DO_REALM_CONVERSION},
-    {"sis",	"sis",		DO_REALM_CONVERSION},
-    {"rfs",	"rfs",		DO_REALM_CONVERSION},
-    {"imap",	"imap",		DO_REALM_CONVERSION},
-    {"ftp",	"ftp",		DO_REALM_CONVERSION},
-    {"ecat",	"ecat",		DO_REALM_CONVERSION},
-    {"daemon",        "daemon",       DO_REALM_CONVERSION},
-    {"gnats", "gnats",        DO_REALM_CONVERSION},
-    {"moira", "moira",        DO_REALM_CONVERSION},
-    {"prms",  "prms",         DO_REALM_CONVERSION},
-    {"mandarin",      "mandarin",     DO_REALM_CONVERSION},
-    {"register",      "register",     DO_REALM_CONVERSION},
-    {"changepw",      "changepw",     DO_REALM_CONVERSION},
-    {"sms",   "sms",          DO_REALM_CONVERSION},
-    {"afpserver",     "afpserver",    DO_REALM_CONVERSION},
-    {"gdss",  "gdss",         DO_REALM_CONVERSION},
-    {"news",  "news",         DO_REALM_CONVERSION},
-    {"abs",   "abs",          DO_REALM_CONVERSION},
-    {"nfs",   "nfs",          DO_REALM_CONVERSION},
-    {"tftp",  "tftp",         DO_REALM_CONVERSION},
-    {"zephyr",        "zephyr",       0},
-    {"http",  "http",         DO_REALM_CONVERSION},
-    {"khttp", "khttp",        DO_REALM_CONVERSION},
-    {"pgpsigner", "pgpsigner",        DO_REALM_CONVERSION},
-    {"irc",   "irc",          DO_REALM_CONVERSION},
-    {"mandarin-agent",        "mandarin-agent",       DO_REALM_CONVERSION},
-    {"write", "write",        DO_REALM_CONVERSION},
-    {"palladium", "palladium",        DO_REALM_CONVERSION},
-    {0,		0,		0},
+    /* Realm conversion, Change service name */
+#define RC(V5NAME,V4NAME) { V5NAME, V4NAME, DO_REALM_CONVERSION, sizeof(V5NAME)-1 }
+    /* Realm conversion */
+#define R(NAME)		{ NAME, NAME, DO_REALM_CONVERSION, sizeof(NAME)-1 }
+    /* No Realm conversion */
+#define NR(NAME)	{ NAME, NAME, 0, sizeof(NAME)-1 }
+
+    NR("kadmin"),
+    RC("rcmd", "host"),
+    R("discuss"),
+    R("rvdsrv"),
+    R("sample"),
+    R("olc"),
+    R("pop"),
+    R("sis"),
+    R("rfs"),
+    R("imap"),
+    R("ftp"),
+    R("ecat"),
+    R("daemon"),
+    R("gnats"),
+    R("moira"),
+    R("prms"),
+    R("mandarin"),
+    R("register"),
+    R("changepw"),
+    R("sms"),
+    R("afpserver"),
+    R("gdss"),
+    R("news"),
+    R("abs"),
+    R("nfs"),
+    R("tftp"),
+    NR("zephyr"),
+    R("http"),
+    R("khttp"),
+    R("pgpsigner"),
+    R("irc"),
+    R("mandarin-agent"),
+    R("write"),
+    R("palladium"),
+    {0, 0, 0, 0},
+#undef R
+#undef RC
+#undef NR
 };
 
 /*
  * char *strnchr(s, c, n)
  *   char *s;
  *   char c;
- *   int n;
+ *   unsigned int n;
  *
  * returns a pointer to the first occurrence of character c in the
  * string s, or a NULL pointer if c does not occur in in the string;
@@ -108,9 +128,8 @@ static const struct krb_convert sconv_list[] = {
  * This falls in the "should have been in the ANSI C library"
  * category. :-)
  */
-static char *strnchr(s, c, n)
-   register char *s, c;
-   register int n;
+static char *strnchr(register char *s, register char c, 
+		     register unsigned int n)
 {
      if (n < 1) 
 	  return 0;
@@ -127,18 +146,15 @@ static char *strnchr(s, c, n)
 /* XXX This calls for a new error code */
 #define KRB5_INVALID_PRINCIPAL KRB5_LNAME_BADFORMAT
 
-KRB5_DLLIMP krb5_error_code KRB5_CALLCONV
-krb5_524_conv_principal(context, princ, name, inst, realm)
-    krb5_context context;
-    const krb5_principal princ;
-    char FAR *name;
-    char FAR *inst;
-    char FAR *realm;
+krb5_error_code KRB5_CALLCONV
+krb5_524_conv_principal(krb5_context context, krb5_const_principal princ,
+			char *name, char *inst, char *realm)
 {
      const struct krb_convert *p;
-     krb5_data *compo;
+     const krb5_data *compo;
      char *c, *tmp_realm, *tmp_prealm;
-     int tmp_realm_len, retval; 
+     unsigned int tmp_realm_len;
+     int retval; 
 
      *name = *inst = '\0';
      switch (krb5_princ_size(context, princ)) {
@@ -147,8 +163,8 @@ krb5_524_conv_principal(context, princ, name, inst, realm)
 	  compo = krb5_princ_component(context, princ, 0);
 	  p = sconv_list;
 	  while (p->v4_str) {
-	       if (strncmp(p->v5_str, compo->data, compo->length) == 0 && 
-	           strlen(p->v5_str) == compo->length) {
+	       if (p->len == compo->length
+		   && memcmp(p->v5_str, compo->data, compo->length) == 0) {
 		   /*
 		    * It is, so set the new name now, and chop off
 		    * instance's domain name if requested.
@@ -161,7 +177,7 @@ krb5_524_conv_principal(context, princ, name, inst, realm)
 		       c = strnchr(compo->data, '.', compo->length);
 		       if (!c || (c - compo->data) >= INST_SZ - 1)
 			   return KRB5_INVALID_PRINCIPAL;
-		       memcpy(inst, compo->data, c - compo->data);
+		       memcpy(inst, compo->data, (size_t) (c - compo->data));
 		       inst[c - compo->data] = '\0';
 		   }
 		   break;
@@ -229,19 +245,14 @@ krb5_524_conv_principal(context, princ, name, inst, realm)
      return 0;
 }
 
-KRB5_DLLIMP krb5_error_code KRB5_CALLCONV
-krb5_425_conv_principal(context, name, instance, realm, princ)
-   krb5_context context;
-   const char	FAR *name;
-   const char	FAR *instance;
-   const char	FAR *realm;
-   krb5_principal	FAR *princ;
+krb5_error_code KRB5_CALLCONV
+krb5_425_conv_principal(krb5_context context, const char *name, const char *instance, const char *realm, krb5_principal *princ)
 {
      const struct krb_convert *p;
      char buf[256];		/* V4 instances are limited to 40 characters */
      krb5_error_code retval;
      char *domain, *cp;
-     char **full_name = 0, **cpp;
+     char **full_name = 0;
      const char *names[5];
      void*	iterator = NULL;
      char** v4realms = NULL;
@@ -273,6 +284,10 @@ krb5_425_conv_principal(context, name, instance, realm, princ)
      	} else if ((retval == 0) && (realm_name == NULL)) {
      		break;
      	}
+	if (v4realms != NULL) {
+	        profile_free_list(v4realms);
+		v4realms = NULL;
+	}
      	if (realm_name != NULL) {
      		profile_release_string (realm_name);
      		realm_name = NULL;
@@ -314,8 +329,8 @@ krb5_425_conv_principal(context, name, instance, realm, princ)
 		      return retval;
 		  if (domain) {
 		      for (cp = domain; *cp; cp++)
-			  if (isupper(*cp))
-			      *cp = tolower(*cp);
+			  if (isupper((int) (*cp)))
+			      *cp = tolower((int) *cp);
 		      strncat(buf, ".", sizeof(buf) - 1 - strlen(buf));
 		      strncat(buf, domain, sizeof(buf) - 1 - strlen(buf));
 		      krb5_xfree(domain);
@@ -328,10 +343,10 @@ krb5_425_conv_principal(context, name, instance, realm, princ)
 not_service:	
      retval = krb5_build_principal(context, princ, strlen(realm), realm, name,
 				   instance, 0);
-     profile_iterator_free (&iterator);
-     profile_free_list(full_name);
-     profile_free_list(v4realms);
-     profile_release_string (realm_name);
-     profile_release_string (dummy_value);
+     if (iterator) profile_iterator_free (&iterator);
+     if (full_name) profile_free_list(full_name);
+     if (v4realms) profile_free_list(v4realms);
+     if (realm_name) profile_release_string (realm_name);
+     if (dummy_value) profile_release_string (dummy_value);
      return retval;
 }
