@@ -28,20 +28,16 @@
 
 #include "k5-int.h"
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_address(context, val)
-    krb5_context context;
-    krb5_address FAR *val;
+void KRB5_CALLCONV
+krb5_free_address(krb5_context context, krb5_address *val)
 {
     if (val->contents)
 	krb5_xfree(val->contents);
     krb5_xfree(val);
 }
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_addresses(context, val)
-    krb5_context context;
-    krb5_address FAR * FAR *val;
+void KRB5_CALLCONV
+krb5_free_addresses(krb5_context context, krb5_address **val)
 {
     register krb5_address **temp;
 
@@ -54,20 +50,16 @@ krb5_free_addresses(context, val)
 }
 
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_ap_rep(context, val)
-    krb5_context context;
-    register krb5_ap_rep FAR *val;
+void KRB5_CALLCONV
+krb5_free_ap_rep(krb5_context context, register krb5_ap_rep *val)
 {
     if (val->enc_part.ciphertext.data)
 	krb5_xfree(val->enc_part.ciphertext.data);
     krb5_xfree(val);
 }
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_ap_req(context, val)
-    krb5_context context;
-    register krb5_ap_req FAR *val;
+void KRB5_CALLCONV
+krb5_free_ap_req(krb5_context context, register krb5_ap_req *val)
 {
     if (val->ticket)
 	krb5_free_ticket(context, val->ticket);
@@ -76,20 +68,16 @@ krb5_free_ap_req(context, val)
     krb5_xfree(val);
 }
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_ap_rep_enc_part(context, val)
-    krb5_context context;
-    krb5_ap_rep_enc_part FAR *val;
+void KRB5_CALLCONV
+krb5_free_ap_rep_enc_part(krb5_context context, krb5_ap_rep_enc_part *val)
 {
     if (val->subkey)
 	krb5_free_keyblock(context, val->subkey);
     krb5_xfree(val);
 }
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_authenticator_contents(context, val)
-    krb5_context context;
-    krb5_authenticator FAR *val;
+void KRB5_CALLCONV
+krb5_free_authenticator_contents(krb5_context context, krb5_authenticator *val)
 {
     if (val->checksum) {
 	krb5_free_checksum(context, val->checksum);
@@ -109,10 +97,8 @@ krb5_free_authenticator_contents(context, val)
     }
 }
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_authdata(context, val)
-    krb5_context context;
-    krb5_authdata FAR * FAR *val;
+void KRB5_CALLCONV
+krb5_free_authdata(krb5_context context, krb5_authdata **val)
 {
     register krb5_authdata **temp;
 
@@ -124,28 +110,22 @@ krb5_free_authdata(context, val)
     krb5_xfree(val);
 }
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_authenticator(context, val)
-    krb5_context context;
-    krb5_authenticator FAR *val;
+void KRB5_CALLCONV
+krb5_free_authenticator(krb5_context context, krb5_authenticator *val)
 {
     krb5_free_authenticator_contents(context, val);
     krb5_xfree(val);
 }
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_checksum(context, val)
-    krb5_context context;
-    register krb5_checksum *val;
+void KRB5_CALLCONV
+krb5_free_checksum(krb5_context context, register krb5_checksum *val)
 {
     krb5_free_checksum_contents(context, val);
     krb5_xfree(val);
 }
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_checksum_contents(context, val)
-    krb5_context context;
-    register krb5_checksum *val;
+void KRB5_CALLCONV
+krb5_free_checksum_contents(krb5_context context, register krb5_checksum *val)
 {
     if (val->contents) {
 	krb5_xfree(val->contents);
@@ -153,10 +133,8 @@ krb5_free_checksum_contents(context, val)
     }
 }
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_cred(context, val)
-    krb5_context context;
-    register krb5_cred FAR *val;
+void KRB5_CALLCONV
+krb5_free_cred(krb5_context context, register krb5_cred *val)
 {
     if (val->tickets)
         krb5_free_tickets(context, val->tickets);
@@ -170,10 +148,8 @@ krb5_free_cred(context, val)
  * the credentials structures 
  */
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_cred_contents(context, val)
-    krb5_context context;
-    krb5_creds FAR *val;
+void KRB5_CALLCONV
+krb5_free_cred_contents(krb5_context context, krb5_creds *val)
 {
     if (val->client) {
 	krb5_free_principal(context, val->client);
@@ -206,10 +182,8 @@ krb5_free_cred_contents(context, val)
     }
 }
 
-KRB5_DLLIMP void KRB5_CALLCONV 
-krb5_free_cred_enc_part(context, val)
-    krb5_context context;
-    register krb5_cred_enc_part FAR *val;
+void KRB5_CALLCONV 
+krb5_free_cred_enc_part(krb5_context context, register krb5_cred_enc_part *val)
 {
     register krb5_cred_info **temp;
     
@@ -240,30 +214,24 @@ krb5_free_cred_enc_part(context, val)
 }
 
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_creds(context, val)
-    krb5_context context;
-    krb5_creds FAR *val;
+void KRB5_CALLCONV
+krb5_free_creds(krb5_context context, krb5_creds *val)
 {
     krb5_free_cred_contents(context, val);
     krb5_xfree(val);
 }
 
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_data(context, val)
-    krb5_context context;
-    krb5_data FAR * val;
+void KRB5_CALLCONV
+krb5_free_data(krb5_context context, krb5_data *val)
 {
     if (val->data)
 	krb5_xfree(val->data);
     krb5_xfree(val);
 }
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_data_contents(context, val)
-    krb5_context context;
-    krb5_data FAR * val;
+void KRB5_CALLCONV
+krb5_free_data_contents(krb5_context context, krb5_data *val)
 {
     if (val->data) {
 	krb5_xfree(val->data);
@@ -271,25 +239,22 @@ krb5_free_data_contents(context, val)
     }
 }
 
-void krb5_free_etype_info(context, info)
-    krb5_context context;
-    krb5_etype_info info;
+void krb5_free_etype_info(krb5_context context, krb5_etype_info info)
 {
   int i;
 
   for(i=0; info[i] != NULL; i++) {
       if (info[i]->salt)
 	  free(info[i]->salt);
+      krb5_free_data_contents( context, &info[i]->s2kparams);
       free(info[i]);
   }
   free(info);
 }
     
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_enc_kdc_rep_part(context, val)
-    krb5_context context;
-    register krb5_enc_kdc_rep_part *val;
+void KRB5_CALLCONV
+krb5_free_enc_kdc_rep_part(krb5_context context, register krb5_enc_kdc_rep_part *val)
 {
     if (val->session)
 	krb5_free_keyblock(context, val->session);
@@ -302,10 +267,8 @@ krb5_free_enc_kdc_rep_part(context, val)
     krb5_xfree(val);
 }
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_enc_tkt_part(context, val)
-    krb5_context context;
-    krb5_enc_tkt_part FAR *val;
+void KRB5_CALLCONV
+krb5_free_enc_tkt_part(krb5_context context, krb5_enc_tkt_part *val)
 {
     if (val->session)
 	krb5_free_keyblock(context, val->session);
@@ -321,10 +284,8 @@ krb5_free_enc_tkt_part(context, val)
 }
 
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_error(context, val)
-    krb5_context context;
-    register krb5_error FAR *val;
+void KRB5_CALLCONV
+krb5_free_error(krb5_context context, register krb5_error *val)
 {
     if (val->client)
 	krb5_free_principal(context, val->client);
@@ -337,10 +298,8 @@ krb5_free_error(context, val)
     krb5_xfree(val);
 }
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_kdc_rep(context, val)
-    krb5_context context;
-    krb5_kdc_rep FAR *val;
+void KRB5_CALLCONV
+krb5_free_kdc_rep(krb5_context context, krb5_kdc_rep *val)
 {
     if (val->padata)
 	krb5_free_pa_data(context, val->padata);
@@ -356,10 +315,8 @@ krb5_free_kdc_rep(context, val)
 }
 
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_kdc_req(context, val)
-    krb5_context context;
-    krb5_kdc_req FAR *val;
+void KRB5_CALLCONV
+krb5_free_kdc_req(krb5_context context, krb5_kdc_req *val)
 {
     if (val->padata)
 	krb5_free_pa_data(context, val->padata);
@@ -380,10 +337,8 @@ krb5_free_kdc_req(context, val)
     krb5_xfree(val);
 }
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_keyblock_contents(context, key)
-     krb5_context context;
-     register krb5_keyblock FAR *key;
+void KRB5_CALLCONV
+krb5_free_keyblock_contents(krb5_context context, register krb5_keyblock *key)
 {
      if (key->contents) {
 	  memset(key->contents, 0, key->length);
@@ -392,10 +347,8 @@ krb5_free_keyblock_contents(context, key)
      }
 }
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_keyblock(context, val)
-    krb5_context context;
-    register krb5_keyblock FAR *val;
+void KRB5_CALLCONV
+krb5_free_keyblock(krb5_context context, register krb5_keyblock *val)
 {
     krb5_free_keyblock_contents(context, val);
     krb5_xfree(val);
@@ -403,10 +356,8 @@ krb5_free_keyblock(context, val)
 
 
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_last_req(context, val)
-    krb5_context context;
-    krb5_last_req_entry FAR * FAR *val;
+void KRB5_CALLCONV
+krb5_free_last_req(krb5_context context, krb5_last_req_entry **val)
 {
     register krb5_last_req_entry **temp;
 
@@ -415,10 +366,8 @@ krb5_free_last_req(context, val)
     krb5_xfree(val);
 }
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_pa_data(context, val)
-    krb5_context context;
-    krb5_pa_data FAR * FAR *val;
+void KRB5_CALLCONV
+krb5_free_pa_data(krb5_context context, krb5_pa_data **val)
 {
     register krb5_pa_data **temp;
 
@@ -430,10 +379,8 @@ krb5_free_pa_data(context, val)
     krb5_xfree(val);
 }
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_principal(context, val)
-    krb5_context context;
-    krb5_principal val;
+void KRB5_CALLCONV
+krb5_free_principal(krb5_context context, krb5_principal val)
 {
     register krb5_int32 i;
 
@@ -451,20 +398,16 @@ krb5_free_principal(context, val)
     krb5_xfree(val);
 }
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_priv(context, val)
-    krb5_context context;
-    register krb5_priv FAR *val;
+void KRB5_CALLCONV
+krb5_free_priv(krb5_context context, register krb5_priv *val)
 {
     if (val->enc_part.ciphertext.data)
 	krb5_xfree(val->enc_part.ciphertext.data);
     krb5_xfree(val);
 }
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_priv_enc_part(context, val)
-    krb5_context context;
-    register krb5_priv_enc_part FAR *val;
+void KRB5_CALLCONV
+krb5_free_priv_enc_part(krb5_context context, register krb5_priv_enc_part *val)
 {
     if (val->user_data.data)
 	krb5_xfree(val->user_data.data);
@@ -475,10 +418,8 @@ krb5_free_priv_enc_part(context, val)
     krb5_xfree(val);
 }
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_pwd_data(context, val)
-    krb5_context context;
-    krb5_pwd_data FAR *val;
+void KRB5_CALLCONV
+krb5_free_pwd_data(krb5_context context, krb5_pwd_data *val)
 {
     if (val->element)
 	krb5_free_pwd_sequences(context, val->element);
@@ -486,26 +427,28 @@ krb5_free_pwd_data(context, val)
 }
 
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_pwd_sequences(context, val)
-    krb5_context context;
-    passwd_phrase_element FAR * FAR *val;
+void KRB5_CALLCONV
+krb5_free_pwd_sequences(krb5_context context, passwd_phrase_element **val)
 {
-    if ((*val)->passwd) {
-	krb5_xfree((*val)->passwd);
-	(*val)->passwd = 0;
-    }
-    if ((*val)->phrase) {
-	krb5_xfree((*val)->phrase);
-	(*val)->phrase = 0;
+    register passwd_phrase_element **temp;
+
+    for (temp = val; *temp; temp++) {
+	if ((*temp)->passwd) {
+	   krb5_free_data(context, (*temp)->passwd);
+	   (*temp)->passwd = 0;
+	}
+	if ((*temp)->phrase) {
+	   krb5_free_data(context, (*temp)->phrase);
+	   (*temp)->phrase = 0;
+	}
+	krb5_xfree(*temp);
     }
+    krb5_xfree(val);
 }
 
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_safe(context, val)
-    krb5_context context;
-    register krb5_safe FAR *val;
+void KRB5_CALLCONV
+krb5_free_safe(krb5_context context, register krb5_safe *val)
 {
     if (val->user_data.data)
 	krb5_xfree(val->user_data.data);
@@ -519,10 +462,8 @@ krb5_free_safe(context, val)
 }
 
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_ticket(context, val)
-    krb5_context context;
-    krb5_ticket FAR *val;
+void KRB5_CALLCONV
+krb5_free_ticket(krb5_context context, krb5_ticket *val)
 {
     if (val->server)
 	krb5_free_principal(context, val->server);
@@ -533,10 +474,8 @@ krb5_free_ticket(context, val)
     krb5_xfree(val);
 }
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_tickets(context, val)
-    krb5_context context;
-    krb5_ticket FAR * FAR *val;
+void KRB5_CALLCONV
+krb5_free_tickets(krb5_context context, krb5_ticket **val)
 {
     register krb5_ticket **temp;
 
@@ -546,10 +485,8 @@ krb5_free_tickets(context, val)
 }
 
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_tgt_creds(context, tgts)
-    krb5_context context;
-    krb5_creds FAR * FAR *tgts;
+void KRB5_CALLCONV
+krb5_free_tgt_creds(krb5_context context, krb5_creds **tgts)
 {
     register krb5_creds **tgtpp;
     for (tgtpp = tgts; *tgtpp; tgtpp++)
@@ -557,10 +494,8 @@ krb5_free_tgt_creds(context, tgts)
     krb5_xfree(tgts);
 }
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_tkt_authent(context, val)
-    krb5_context context;
-    krb5_tkt_authent FAR *val;
+void KRB5_CALLCONV
+krb5_free_tkt_authent(krb5_context context, krb5_tkt_authent *val)
 {
     if (val->ticket)
 	    krb5_free_ticket(context, val->ticket);
@@ -569,17 +504,15 @@ krb5_free_tkt_authent(context, val)
     krb5_xfree(val);
 }
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_unparsed_name(context, val)
-    krb5_context context;
-    char FAR * val;
+void KRB5_CALLCONV
+krb5_free_unparsed_name(krb5_context context, char *val)
 {
     if (val)
 	krb5_xfree(val);
 }
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_sam_challenge(krb5_context ctx, krb5_sam_challenge FAR *sc)
+void KRB5_CALLCONV
+krb5_free_sam_challenge(krb5_context ctx, krb5_sam_challenge *sc)
 {
     if (!sc)
 	return;
@@ -587,8 +520,17 @@ krb5_free_sam_challenge(krb5_context ctx, krb5_sam_challenge FAR *sc)
     krb5_xfree(sc);
 }
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_sam_challenge_contents(krb5_context ctx, krb5_sam_challenge FAR *sc)
+void KRB5_CALLCONV
+krb5_free_sam_challenge_2(krb5_context ctx, krb5_sam_challenge_2 *sc2)
+{
+    if (!sc2)
+	return;
+    krb5_free_sam_challenge_2_contents(ctx, sc2);
+    krb5_xfree(sc2);
+}
+
+void KRB5_CALLCONV
+krb5_free_sam_challenge_contents(krb5_context ctx, krb5_sam_challenge *sc)
 {
     if (!sc)
 	return;
@@ -610,8 +552,59 @@ krb5_free_sam_challenge_contents(krb5_context ctx, krb5_sam_challenge FAR *sc)
     }
 }
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_sam_response(krb5_context ctx, krb5_sam_response FAR *sr)
+void KRB5_CALLCONV
+krb5_free_sam_challenge_2_contents(krb5_context ctx,
+				   krb5_sam_challenge_2 *sc2)
+{
+    krb5_checksum **cksump;
+
+    if (!sc2)
+	return;
+    if (sc2->sam_challenge_2_body.data)
+	krb5_free_data_contents(ctx, &sc2->sam_challenge_2_body);
+    if (sc2->sam_cksum) {
+	cksump = sc2->sam_cksum;
+	while (*cksump) {
+	    krb5_free_checksum(ctx, *cksump);
+	    cksump++;
+	}
+	krb5_xfree(sc2->sam_cksum);
+	sc2->sam_cksum = 0;
+    }
+}
+
+void KRB5_CALLCONV
+krb5_free_sam_challenge_2_body(krb5_context ctx,
+			       krb5_sam_challenge_2_body *sc2)
+{
+    if (!sc2)
+	return;
+    krb5_free_sam_challenge_2_body_contents(ctx, sc2);
+    krb5_xfree(sc2);
+}
+
+void KRB5_CALLCONV
+krb5_free_sam_challenge_2_body_contents(krb5_context ctx,
+					krb5_sam_challenge_2_body *sc2)
+{
+    if (!sc2)
+	return;
+    if (sc2->sam_type_name.data) 
+	krb5_free_data_contents(ctx, &sc2->sam_type_name);
+    if (sc2->sam_track_id.data)
+	krb5_free_data_contents(ctx, &sc2->sam_track_id);
+    if (sc2->sam_challenge_label.data)
+	krb5_free_data_contents(ctx, &sc2->sam_challenge_label);
+    if (sc2->sam_challenge.data)
+	krb5_free_data_contents(ctx, &sc2->sam_challenge);
+    if (sc2->sam_response_prompt.data)
+	krb5_free_data_contents(ctx, &sc2->sam_response_prompt);
+    if (sc2->sam_pk_for_sad.data)
+	krb5_free_data_contents(ctx, &sc2->sam_pk_for_sad);
+}
+
+void KRB5_CALLCONV
+krb5_free_sam_response(krb5_context ctx, krb5_sam_response *sr)
 {
     if (!sr)
 	return;
@@ -619,8 +612,17 @@ krb5_free_sam_response(krb5_context ctx, krb5_sam_response FAR *sr)
     krb5_xfree(sr);
 }
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_sam_response_contents(krb5_context ctx, krb5_sam_response FAR *sr)
+void KRB5_CALLCONV
+krb5_free_sam_response_2(krb5_context ctx, krb5_sam_response_2 *sr2)
+{
+    if (!sr2)
+	return;
+    krb5_free_sam_response_2_contents(ctx, sr2);
+    krb5_xfree(sr2);
+}
+
+void KRB5_CALLCONV
+krb5_free_sam_response_contents(krb5_context ctx, krb5_sam_response *sr)
 {
     if (!sr)
 	return;
@@ -632,9 +634,20 @@ krb5_free_sam_response_contents(krb5_context ctx, krb5_sam_response FAR *sr)
 	krb5_free_data_contents(ctx, &sr->sam_enc_nonce_or_ts.ciphertext);
 }
 
-KRB5_DLLIMP void KRB5_CALLCONV
+void KRB5_CALLCONV
+krb5_free_sam_response_2_contents(krb5_context ctx, krb5_sam_response_2 *sr2)
+{
+    if (!sr2)
+	return;
+    if (sr2->sam_track_id.data)
+	krb5_free_data_contents(ctx, &sr2->sam_track_id);
+    if (sr2->sam_enc_nonce_or_sad.ciphertext.data)
+	krb5_free_data_contents(ctx, &sr2->sam_enc_nonce_or_sad.ciphertext);
+}
+
+void KRB5_CALLCONV
 krb5_free_predicted_sam_response(krb5_context ctx,
-				 krb5_predicted_sam_response FAR *psr)
+				 krb5_predicted_sam_response *psr)
 {
     if (!psr)
 	return;
@@ -642,9 +655,9 @@ krb5_free_predicted_sam_response(krb5_context ctx,
     krb5_xfree(psr);
 }
 
-KRB5_DLLIMP void KRB5_CALLCONV
+void KRB5_CALLCONV
 krb5_free_predicted_sam_response_contents(krb5_context ctx,
-				 krb5_predicted_sam_response FAR *psr)
+				 krb5_predicted_sam_response *psr)
 {
     if (!psr)
 	return;
@@ -658,9 +671,9 @@ krb5_free_predicted_sam_response_contents(krb5_context ctx,
 	krb5_free_data_contents(ctx, &psr->msd);
 }
 
-KRB5_DLLIMP void KRB5_CALLCONV
+void KRB5_CALLCONV
 krb5_free_enc_sam_response_enc(krb5_context ctx,
-			       krb5_enc_sam_response_enc FAR *esre)
+			       krb5_enc_sam_response_enc *esre)
 {
     if (!esre)
 	return;
@@ -668,9 +681,19 @@ krb5_free_enc_sam_response_enc(krb5_context ctx,
     krb5_xfree(esre);
 }
 
-KRB5_DLLIMP void KRB5_CALLCONV
+void KRB5_CALLCONV 
+krb5_free_enc_sam_response_enc_2(krb5_context ctx,
+				 krb5_enc_sam_response_enc_2 *esre2)
+{
+    if (!esre2)
+	return;
+    krb5_free_enc_sam_response_enc_2_contents(ctx, esre2);
+    krb5_xfree(esre2);
+}
+
+void KRB5_CALLCONV
 krb5_free_enc_sam_response_enc_contents(krb5_context ctx,
-			       krb5_enc_sam_response_enc FAR *esre)
+			       krb5_enc_sam_response_enc *esre)
 {
     if (!esre)
 	return;
@@ -678,8 +701,18 @@ krb5_free_enc_sam_response_enc_contents(krb5_context ctx,
 	krb5_free_data_contents(ctx, &esre->sam_sad);
 }
 
-KRB5_DLLIMP void KRB5_CALLCONV
-krb5_free_pa_enc_ts(krb5_context ctx, krb5_pa_enc_ts FAR *pa_enc_ts)
+void KRB5_CALLCONV
+krb5_free_enc_sam_response_enc_2_contents(krb5_context ctx,
+					  krb5_enc_sam_response_enc_2 *esre2)
+{
+    if (!esre2)
+	return;
+    if (esre2->sam_sad.data)
+	krb5_free_data_contents(ctx, &esre2->sam_sad);
+}
+
+void KRB5_CALLCONV
+krb5_free_pa_enc_ts(krb5_context ctx, krb5_pa_enc_ts *pa_enc_ts)
 {
     if (!pa_enc_ts)
 	return;
