@@ -30,24 +30,25 @@
 #define NEED_SOCKETS
 #include "k5-int.h"
 #include "auth_con.h"
+#include "com_err.h"
 #include <errno.h>
 #include <stdio.h>
 #include <string.h>
 
-static char *sendauth_version = "KRB5_SENDAUTH_V1.0";
+static const char sendauth_version[] = "KRB5_SENDAUTH_V1.0";
 
-krb5_error_code
+static krb5_error_code
 recvauth_common(krb5_context context,
-		krb5_auth_context FAR * auth_context,
+		krb5_auth_context * auth_context,
 		/* IN */
 		krb5_pointer fd,
-		char FAR *appl_version,
+		char *appl_version,
 		krb5_principal server,
 		krb5_int32 flags,
 		krb5_keytab keytab,
 		/* OUT */
-		krb5_ticket FAR * FAR * ticket,
-		krb5_data FAR *version)
+		krb5_ticket ** ticket,
+		krb5_data *version)
 {
     krb5_auth_context	  new_auth_context;
     krb5_flags		  ap_option;
@@ -244,36 +245,24 @@ cleanup:;
     return retval;
 }
 
-KRB5_DLLIMP krb5_error_code KRB5_CALLCONV
-krb5_recvauth(context, auth_context,
-	      /* IN */
-	      fd, appl_version, server, flags, keytab,
-	      /* OUT */
-	      ticket)
-    krb5_context 	  context;
-    krb5_auth_context   FAR * auth_context;
-    krb5_pointer	  fd;
-    char		FAR * appl_version;
-    krb5_principal	  server;
-    krb5_int32		  flags;
-    krb5_keytab		  keytab;
-    krb5_ticket	       FAR * FAR * ticket;
+krb5_error_code KRB5_CALLCONV
+krb5_recvauth(krb5_context context, krb5_auth_context *auth_context, krb5_pointer fd, char *appl_version, krb5_principal server, krb5_int32 flags, krb5_keytab keytab, krb5_ticket **ticket)
 {
     return recvauth_common (context, auth_context, fd, appl_version,
 			    server, flags, keytab, ticket, 0);
 }
 
-KRB5_DLLIMP krb5_error_code KRB5_CALLCONV
+krb5_error_code KRB5_CALLCONV
 krb5_recvauth_version(krb5_context context,
-		      krb5_auth_context FAR *auth_context,
+		      krb5_auth_context *auth_context,
 		      /* IN */
 		      krb5_pointer fd,
 		      krb5_principal server,
 		      krb5_int32 flags,
 		      krb5_keytab keytab,
 		      /* OUT */
-		      krb5_ticket FAR * FAR *ticket,
-		      krb5_data FAR *version)
+		      krb5_ticket **ticket,
+		      krb5_data *version)
 {
     return recvauth_common (context, auth_context, fd, 0,
 			    server, flags, keytab, ticket, version);
