@@ -63,8 +63,7 @@
  * Find the portion of the flattened principal name that we use for mapping.
  */
 static char *
-aname_full_to_mapping_name(fprincname)
-    char *fprincname;
+aname_full_to_mapping_name(char *fprincname)
 {
     char	*atp;
     size_t	mlen;
@@ -98,10 +97,10 @@ db_an_to_ln(context, dbname, aname, lnsize, lname)
     krb5_context context;
     char *dbname;
     krb5_const_principal aname;
-    const int lnsize;
+    const unsigned int lnsize;
     char *lname;
 {
-#if	(!defined(_MSDOS) && !defined(_WIN32) && !defined(macintosh))
+#if	(!defined(_WIN32) && !defined(macintosh))
     DBM *db;
     krb5_error_code retval;
     datum key, contents;
@@ -137,13 +136,13 @@ db_an_to_ln(context, dbname, aname, lnsize, lname)
     /* can't close until we copy the contents. */
     (void) KDBM_CLOSE(db);
     return retval;
-#else	/* !_MSDOS && !_WIN32 && !MACINTOSH */
+#else	/* !_WIN32 && !MACINTOSH */
     /*
      * If we don't have support for a database mechanism, then we can't
      * translate this now, can we?
      */
     return KRB5_LNAME_NOTRANS;
-#endif	/* !_MSDOS && !_WIN32 && !MACINTOSH */
+#endif	/* !_WIN32 && !MACINTOSH */
 }
 #endif /*ANAME_DB*/
 
@@ -196,9 +195,7 @@ db_an_to_ln(context, dbname, aname, lnsize, lname)
  * If no re_comp() or regcomp(), then always return a match.
  */
 static krb5_error_code
-aname_do_match(string, contextp)
-    char	*string;
-    char	**contextp;
+aname_do_match(char *string, char **contextp)
 {
     krb5_error_code	kret;
     char		*regexp, *startp, *endp = 0;
@@ -273,14 +270,14 @@ aname_do_match(string, contextp)
  * If no regcomp() then just return the input string verbatim in the output
  * string.
  */
-static void
-do_replacement(regexp, repl, doall, in, out)
-    char	*regexp;
-    char	*repl;
-    int		doall;
-    char	*in;
-    char	*out;
+#define use_bytes(x) \
+    out_used += (x); \
+    if (out_used > MAX_FORMAT_BUFFER) goto mem_err
+
+static int
+do_replacement(char *regexp, char *repl, int doall, char *in, char *out)
 {
+    size_t out_used = 0;
 #if	HAVE_REGCOMP
     regex_t	match_exp;
     regmatch_t	match_match;
@@ -295,17 +292,22 @@ do_replacement(regexp, repl, doall, in, out)
 	do {
 	    if (!regexec(&match_exp, cp, 1, &match_match, 0)) {
 		if (match_match.rm_so) {
+		    use_bytes(match_match.rm_so);
 		    strncpy(op, cp, match_match.rm_so);
 		    op += match_match.rm_so;
 		}
+		use_bytes(strlen(repl));
 		strncpy(op, repl, MAX_FORMAT_BUFFER - 1 - (op - out));
 		op += strlen(op);
 		cp += match_match.rm_eo;
-		if (!doall)
+		if (!doall) {
+		    use_bytes(strlen(cp));
 		    strncpy(op, cp, MAX_FORMAT_BUFFER - 1 - (op - out));
+		}
 		matched = 1;
 	    }
 	    else {
+		use_bytes(strlen(cp));
 		strncpy(op, cp, MAX_FORMAT_BUFFER - 1 - (op - out));
 		matched = 0;
 	    }
@@ -330,17 +332,22 @@ do_replacement(regexp, repl, doall, in, out)
 	    sdispl = (size_t) (loc1 - cp);
 	    edispl = (size_t) (loc2 - cp);
 	    if (sdispl) {
+		use_bytes(sdispl);
 		strncpy(op, cp, sdispl);
 		op += sdispl;
 	    }
+	    use_bytes(strlen(repl));
 	    strncpy(op, repl, MAX_FORMAT_BUFFER - 1 - (op - out));
 	    op += strlen(repl);
 	    cp += edispl;
-	    if (!doall)
+	    if (!doall) {
+		use_bytes(strlen(cp));
 		strncpy(op, cp, MAX_FORMAT_BUFFER - 1 - (op - out));
+	    }
 	    matched = 1;
 	}
 	else {
+	    use_bytes(strlen(cp));
 	    strncpy(op, cp, MAX_FORMAT_BUFFER - 1 - (op - out));
 	    matched = 0;
 	}
@@ -348,7 +355,15 @@ do_replacement(regexp, repl, doall, in, out)
 #else	/* HAVE_REGEXP_H */
     memcpy(out, in, MAX_FORMAT_BUFFER);
 #endif	/* HAVE_REGCOMP */
+    return 1;
+ mem_err:
+#ifdef HAVE_REGCMP
+    	regfree(&match_exp);
+#endif
+	return 0;
+	
 }
+#undef use_bytes
 
 /*
  * aname_replacer()	- Perform the specified substitutions on the input
@@ -357,10 +372,7 @@ do_replacement(regexp, repl, doall, in, out)
  * This routine enforces the "s/<pattern>/<replacement>/[g]" syntax.
  */
 static krb5_error_code
-aname_replacer(string, contextp, result)
-    char	*string;
-    char	**contextp;
-    char	**result;
+aname_replacer(char *string, char **contextp, char **result)
 {
     krb5_error_code	kret;
     char		*in;
@@ -388,7 +400,7 @@ aname_replacer(string, contextp, result)
 	 */
 	for (cp = *contextp; *cp; ) {
 	    /* Skip leading whitespace */
-	    while (isspace(*cp))
+	    while (isspace((int) (*cp)))
 		cp++;
 
 	    /*
@@ -423,7 +435,12 @@ aname_replacer(string, contextp, result)
 
 		    /* Do the replacemenbt */
 		    memset(out, '\0', MAX_FORMAT_BUFFER);
-		    do_replacement(rule, repl, doglobal, in, out);
+		    if (!do_replacement(rule, repl, doglobal, in, out)) {
+			free(rule);
+		    free(repl);
+			kret = KRB5_LNAME_NOTRANS;
+			break;
+		    }
 		    free(rule);
 		    free(repl);
 
@@ -463,20 +480,16 @@ aname_replacer(string, contextp, result)
  * the principal name.
  */
 static krb5_error_code
-rule_an_to_ln(context, rule, aname, lnsize, lname)
-    krb5_context		context;
-    char *			rule;
-    krb5_const_principal	aname;
-    const int			lnsize;
-    char *			lname;
+rule_an_to_ln(krb5_context context, char *rule, krb5_const_principal aname, const unsigned int lnsize, char *lname)
 {
     krb5_error_code	kret;
     char		*current;
     char		*fprincname;
     char		*selstring = 0;
     int			num_comps, compind;
+    size_t selstring_used;
     char		*cout;
-    krb5_data		*datap;
+    krb5_const krb5_data *datap;
     char		*outstring;
 
     /*
@@ -495,6 +508,7 @@ rule_an_to_ln(context, rule, aname, lnsize, lname)
 		     */
 		    current = strchr(current, ':');
 		    selstring = (char *) malloc(MAX_FORMAT_BUFFER);
+		    selstring_used = 0;
 		    if (current && selstring) {
 			current++;
 			cout = selstring;
@@ -509,18 +523,28 @@ rule_an_to_ln(context, rule, aname, lnsize, lname)
 			    if (*current == '$') {
 				if ((sscanf(current+1, "%d", &compind) == 1) &&
 				    (compind <= num_comps) &&
-				    (datap = krb5_princ_component(context,
-								  aname,
-								  compind-1))
+				    (datap =
+				     (compind > 0)
+				     ? krb5_princ_component(context, aname,
+							    compind-1)
+				     : krb5_princ_realm(context, aname))
 				    ) {
+				    if ((datap->length < MAX_FORMAT_BUFFER)
+					&&  (selstring_used+datap->length
+					     < MAX_FORMAT_BUFFER)) {
+					selstring_used += datap->length;
+				    } else {
+					kret = ENOMEM;
+					goto errout;
+				    }
 				    strncpy(cout,
 					    datap->data,
-					    datap->length);
+					    (unsigned) datap->length);
 				    cout += datap->length;
 				    *cout = '\0';
 				    current++;
 				    /* Point past number */
-				    while (isdigit(*current))
+				    while (isdigit((int) (*current)))
 					current++;
 				}
 				else
@@ -543,7 +567,7 @@ rule_an_to_ln(context, rule, aname, lnsize, lname)
 			else
 			    kret = KRB5_CONFIG_BADFORMAT;
 
-			if (kret)
+			errout: if (kret)
 			    free(selstring);
 		    }
 		}
@@ -594,15 +618,11 @@ rule_an_to_ln(context, rule, aname, lnsize, lname)
  * that name is returned as the lname.
  */
 static krb5_error_code
-default_an_to_ln(context, aname, lnsize, lname)
-    krb5_context context;
-    krb5_const_principal aname;
-    const int lnsize;
-    char *lname;
+default_an_to_ln(krb5_context context, krb5_const_principal aname, const unsigned int lnsize, char *lname)
 {
     krb5_error_code retval;
     char *def_realm;
-    int realm_length;
+    unsigned int realm_length;
 
     realm_length = krb5_princ_realm(context, aname)->length;
     
@@ -653,12 +673,8 @@ default_an_to_ln(context, aname, lnsize, lname)
  returns system errors, NOT_ENOUGH_SPACE
 */
 
-krb5_error_code
-krb5_aname_to_localname(context, aname, lnsize, lname)
-    krb5_context context;
-    krb5_const_principal aname;
-    const int lnsize;
-    char *lname;
+krb5_error_code KRB5_CALLCONV
+krb5_aname_to_localname(krb5_context context, krb5_const_principal aname, const int lnsize_in, char *lname)
 {
     krb5_error_code	kret;
     char		*realm;
@@ -667,8 +683,14 @@ krb5_aname_to_localname(context, aname, lnsize, lname)
     const char		*hierarchy[5];
     char		**mapping_values;
     int			i, nvalid;
-    char		*cp;
+    char		*cp, *s;
     char		*typep, *argp;
+    unsigned int        lnsize;
+
+    if (lnsize_in < 0)
+      return KRB5_CONFIG_NOTENUFSPACE;
+
+    lnsize = lnsize_in; /* Unsigned */
 
     /*
      * First get the default realm.
@@ -695,11 +717,14 @@ krb5_aname_to_localname(context, aname, lnsize, lname)
 
 		    /* Just use the last one. */
 		    /* Trim the value. */
-		    cp = &mapping_values[nvalid-1]
-			[strlen(mapping_values[nvalid-1])];
-		    while (isspace(*cp)) cp--;
-		    cp++;
-		    *cp = '\0';
+		    s = mapping_values[nvalid-1];
+		    cp = s + strlen(s);
+		    while (cp > s) {
+			cp--;
+			if (!isspace((int)(*cp)))
+			    break;
+			*cp = '\0';
+		    }
 
 		    /* Copy out the value if there's enough room */
 		    if (strlen(mapping_values[nvalid-1])+1 <= (size_t) lnsize)
