@@ -53,12 +53,12 @@ static struct svcraw_private {
 	char	verf_body[MAX_AUTH_BYTES];
 } *svcraw_private;
 
-static bool_t		svcraw_recv();
-static enum xprt_stat 	svcraw_stat();
-static bool_t		svcraw_getargs();
-static bool_t		svcraw_reply();
-static bool_t		svcraw_freeargs();
-static void		svcraw_destroy();
+static bool_t		svcraw_recv(SVCXPRT *, struct rpc_msg *);
+static enum xprt_stat 	svcraw_stat(SVCXPRT *);
+static bool_t		svcraw_getargs(SVCXPRT *, xdrproc_t, void *);
+static bool_t		svcraw_reply(SVCXPRT *, struct rpc_msg *);
+static bool_t		svcraw_freeargs(SVCXPRT *, xdrproc_t, void *);
+static void		svcraw_destroy(SVCXPRT *);
 
 static struct xp_ops server_ops = {
 	svcraw_recv,
@@ -88,7 +88,8 @@ svcraw_create()
 }
 
 static enum xprt_stat
-svcraw_stat()
+svcraw_stat(xprt)
+	SVCXPRT *xprt;
 {
 
 	return (XPRT_IDLE);
@@ -135,7 +136,7 @@ static bool_t
 svcraw_getargs(xprt, xdr_args, args_ptr)
 	SVCXPRT *xprt;
 	xdrproc_t xdr_args;
-	caddr_t args_ptr;
+	void * args_ptr;
 {
 	register struct svcraw_private *srp = svcraw_private;
 
@@ -152,7 +153,7 @@ static bool_t
 svcraw_freeargs(xprt, xdr_args, args_ptr)
 	SVCXPRT *xprt;
 	xdrproc_t xdr_args;
-	caddr_t args_ptr;
+	void * args_ptr;
 { 
 	register struct svcraw_private *srp = svcraw_private;
 	register XDR *xdrs;
@@ -165,6 +166,7 @@ svcraw_freeargs(xprt, xdr_args, args_ptr)
 } 
 
 static void
-svcraw_destroy()
+svcraw_destroy(xprt)
+	SVCXPRT *xprt;
 {
 }
