@@ -22,7 +22,7 @@
  * 
  */
 
-#include <com_err.h>
+#include "com_err.h"
 #include "libpty.h"
 #include "pty-int.h"
 
