@@ -41,7 +41,9 @@ HFONT hfontdialog = NULL;	       /* Font in which the dialog is drawn. */
 static HFONT hfonticon = NULL;	       /* Font for icon label */
 HINSTANCE hinstance;
 static int dlgncmdshow;		       /* ncmdshow from WinMain */
+#if 0
 static UINT wm_kerberos_changed;       /* message for cache changing */
+#endif
 static int action;		       /* After login actions */
 static UINT kwin_timer_id;	       /* Timer being used for update */
 BOOL alert;		       	       /* Actions on ticket expiration */
@@ -56,6 +58,7 @@ char confname[FILENAME_MAX];           /* krb5.conf (or krb.conf for krb4) */
 #ifdef KRB5
 char ccname[FILENAME_MAX];             /* ccache file location */
 BOOL forwardable;                      /* TRUE to get forwardable tickets */
+BOOL noaddresses;
 krb5_context k5_context;
 krb5_ccache k5_ccache;
 #endif
@@ -1222,7 +1225,10 @@ kwin_command(HWND hwnd, int cid, HWND hwndCtl, UINT codeNotify)
     krb5_get_init_creds_opt_init(&opts);
     krb5_get_init_creds_opt_set_forwardable(&opts, forwardable);
     krb5_get_init_creds_opt_set_tkt_life(&opts, lifetime * 60);
-    
+    if (noaddresses) {
+		krb5_get_init_creds_opt_set_address_list(&opts, NULL);
+ 	}    
+
     /*
      * get the initial creds using the password and the options we set above
      */
@@ -1489,14 +1495,15 @@ kwin_paint(HWND hwnd)
 LRESULT CALLBACK
 kwin_wnd_proc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 {
-  int n;
 
+#if 0
   if (message == wm_kerberos_changed) {       /* Message from the ccache */
     n = ticket_init_list(GetDlgItem(hwnd, IDD_TICKET_LIST));
     EnableWindow(GetDlgItem(hwnd, IDD_TICKET_DELETE), n > 0);
 
     return 0;
   }
+#endif
 
   switch (message) {
     HANDLE_MSG(hwnd, WM_GETMINMAXINFO, kwin_getminmaxinfo);
@@ -1601,12 +1608,14 @@ init_application(HINSTANCE hinstance)
 {
   BOOL rc;
 
+#if 0
 #ifdef KRB4
   wm_kerberos_changed = krb_get_notification_message();
 #endif
 
 #ifdef KRB5
   wm_kerberos_changed = krb5_get_notification_message();
+#endif
 #endif
 
   rc = kwin_init(hinstance);
@@ -1698,6 +1707,7 @@ init_instance(HINSTANCE hinstance, int ncmdshow)
    * ticket options
    */
   forwardable = cns_res.forwardable;
+  noaddresses = cns_res.noaddresses;
 
   /*
    * Load clock icons
@@ -1916,9 +1926,9 @@ krb_gethostbyname_wnd_proc(HWND hwnd, UINT message,
  * the async routines to emulate the functionality of the synchronous
  * routines
  */
-struct hostent FAR *PASCAL FAR
+struct hostent *PASCAL
 krb_gethostbyname(
-		  const char FAR *name)
+		  const char *name)
 {
   HWND hwnd;
   char buf[MAXGETHOSTSTRUCT];
